@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -32,9 +32,8 @@ from bigmler.analyze.k_fold_cv import (create_kfold_cv,
                                        create_features_analysis,
                                        create_nodes_analysis,
                                        create_candidates_analysis)
-from bigmler.dispatcher import (SESSIONS_LOG, command_handling,
-                                clear_log_files)
-from bigmler.command import get_stored_command
+from bigmler.dispatcher import SESSIONS_LOG, clear_log_files
+from bigmler.command import get_stored_command, command_handling
 
 
 COMMAND_LOG = u".bigmler_analyze"
@@ -108,22 +107,22 @@ def analyze_dispatcher(args=sys.argv[1:]):
                  "threshold, cross validation or random fields.")
     # k-fold cross-validation
     if command_args.cv and command_args.dataset is not None:
-        create_kfold_cv(command_args, api, command.common_options,
+        create_kfold_cv(command_args, api, command,
                         resume=resume)
 
     # features analysis
     elif command_args.features:
-        create_features_analysis(command_args, api, command.common_options,
+        create_features_analysis(command_args, api, command,
                                  resume=resume)
 
     # node threshold analysis
     elif command_args.nodes:
-        create_nodes_analysis(command_args, api, command.common_options,
+        create_nodes_analysis(command_args, api, command,
                               resume=resume)
 
     # random fields analysis
     elif command_args.random_fields:
-        create_candidates_analysis(command_args, api, command.common_options,
+        create_candidates_analysis(command_args, api, command,
                                    resume=resume)
     else:
         sys.exit("You must choose one of the available analysis: --features,"
