@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2016 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -28,16 +28,16 @@ import sys
 import json
 import re
 
-import bigml
-
-import bigmler.processing.args as a
-import bigmler.utils as u
-
 from copy import copy
 
+import bigml
+
 from bigml.fields import Fields
 from bigml.io import UnicodeWriter
 
+import bigmler.processing.args as a
+import bigmler.utils as u
+
 from bigmler.dispatcher import main_dispatcher
 from bigmler.options.analyze import ACCURACY, MINIMIZE_OPTIONS
 from bigmler.resources import ALL_FIELDS_QS
@@ -133,7 +133,6 @@ def set_subcommand_file(output_dir):
                                                     SUBCOMMAND_LOG))
     session_file = os.path.normpath(os.path.join(output_dir, SESSIONS_LOG))
 
-
 def retrieve_subcommands():
     """Retrieves the executed subcommands in inverse order
 
@@ -172,7 +171,6 @@ def create_prediction_dataset(base_path, folder, args, resume):
 
     """
     args.output_dir = os.path.join(base_path, "%s_pred" % folder)
-    output_dir = args.output_dir
     folder = os.path.join(base_path, folder)
     model_type = "ensembles" if hasattr(args, "number_of_models") and \
         args.number_of_models > 1 else "models"
@@ -196,7 +194,7 @@ def create_prediction_dataset(base_path, folder, args, resume):
     return resume
 
 
-def create_kfold_cv(args, api, common_options, resume=False):
+def create_kfold_cv(args, api, command_obj, resume=False):
     """Creates the kfold cross-validation
 
     """
@@ -204,7 +202,7 @@ def create_kfold_cv(args, api, common_options, resume=False):
     if resume:
         retrieve_subcommands()
     datasets_file, objective_name, resume = create_kfold_datasets_file(
-        args, api, common_options, resume=resume)
+        args, api, command_obj, resume=resume)
     if datasets_file is not None:
         args.output_dir = os.path.normpath(
             os.path.join(u.check_dir(datasets_file),
@@ -213,11 +211,11 @@ def create_kfold_cv(args, api, common_options, resume=False):
         u.log_message(message, log_file=session_file,
                       console=args.verbosity)
         args.objective_field = objective_name
-        create_kfold_evaluations(datasets_file, args, common_options,
+        create_kfold_evaluations(datasets_file, args, command_obj,
                                  resume=resume)
 
 
-def create_features_analysis(args, api, common_options, resume=False):
+def create_features_analysis(args, api, command_obj, resume=False):
     """Analyzes the features in the dataset to find the ones that
        optimize the model performance
 
@@ -227,13 +225,13 @@ def create_features_analysis(args, api, common_options, resume=False):
     if resume:
         retrieve_subcommands()
     datasets_file, objective_name, resume = create_kfold_datasets_file(
-        args, api, common_options, resume=resume)
+        args, api, command_obj, resume=resume)
     args.objective_field = objective_name
     message = ('Creating the best features set..........\n')
     u.log_message(message, log_file=session_file,
                   console=args.verbosity)
     model_fields = best_first_search(
-        datasets_file, api, args, common_options,
+        datasets_file, api, args, command_obj,
         staleness=args.staleness,
         penalty=args.penalty,
         objective_name=objective_name, resume=resume)
@@ -245,6 +243,8 @@ def create_features_analysis(args, api, common_options, resume=False):
     if args.number_of_models > 1:
         bigmler_command = u"%s --number-of-models %s" % ( \
             bigmler_command, args.number_of_models)
+    connection_args = command_obj.propagate([], connection_only=True)
+    bigmler_command = u"%s %s" % (bigmler_command, " ".join(connection_args))
     message = (u'To create the final model with the entire dataset using '
                u'the selected feature subset use:\n%s\n\n' %
                bigmler_command)
@@ -252,13 +252,14 @@ def create_features_analysis(args, api, common_options, resume=False):
     # showing the instruction to delete the user-given output-dir
     bigmler_command = (u'bigmler delete --from-dir %s') % (
         output_dir)
+    bigmler_command = u"%s %s" % (bigmler_command, " ".join(connection_args))
     message = (u'To delete all the resources generated by this analyze'
                u' subcommand use:\n%s\n\n') % bigmler_command
     u.log_message(message, log_file=session_file, console=1)
 
 
 
-def create_nodes_analysis(args, api, common_options, resume=False):
+def create_nodes_analysis(args, api, command_obj, resume=False):
     """Analyzes the model performance as a function of node threshold.
 
     """
@@ -267,13 +268,13 @@ def create_nodes_analysis(args, api, common_options, resume=False):
     if resume:
         retrieve_subcommands()
     datasets_file, objective_name, resume = create_kfold_datasets_file(
-        args, api, common_options, resume=resume)
+        args, api, command_obj, resume=resume)
     args.objective_field = objective_name
     message = ('Creating the node threshold set..........\n')
     u.log_message(message, log_file=session_file,
                   console=args.verbosity)
     node_threshold = best_node_threshold(
-        datasets_file, args, common_options,
+        datasets_file, args, command_obj,
         staleness=args.staleness,
         penalty=args.penalty,
         resume=resume)
@@ -293,7 +294,7 @@ def create_nodes_analysis(args, api, common_options, resume=False):
     u.log_message(message, log_file=session_file, console=1)
 
 
-def create_kfold_datasets_file(args, api, common_options, resume=False):
+def create_kfold_datasets_file(args, api, command_obj, resume=False):
     """Create the kfold dataset resources and store their ids in a file
        one per line
 
@@ -359,7 +360,7 @@ def create_kfold_datasets_file(args, api, common_options, resume=False):
         # generate test datasets
         datasets_file, resume = create_kfold_datasets(dataset_id, args,
                                                       selecting_file_list,
-                                                      common_options,
+                                                      command_obj,
                                                       resume=resume)
         return datasets_file, objective_name, resume
     return None, None, None
@@ -405,7 +406,7 @@ def avoid_duplicates(field_name, fields, affix="_"):
 
 def create_kfold_datasets(dataset, args,
                           selecting_file_list,
-                          common_options, resume=False):
+                          command_obj, resume=False):
     """Calling the bigmler procedure to create the k-fold datasets
 
     """
@@ -418,9 +419,7 @@ def create_kfold_datasets(dataset, args,
             dataset, selecting_file_list[index],
             output_dir)
         command_args = command.split()
-        common_options_list = u.get_options_list(args, common_options,
-                                                 prioritary=command_args)
-        command_args.extend(common_options_list)
+        command_obj.propagate(command_args)
         command = rebuild_command(command_args)
         if resume:
             next_command = subcommand_list.pop()
@@ -455,9 +454,6 @@ def add_model_options(command_args, args):
     if args.objective_weights:
         command_args.append("--objective-weights")
         command_args.append(args.objective_weights)
-    if args.tlp:
-        command_args.append("--tlp")
-        command_args.append(str(args.tlp))
     if args.model_attributes:
         command_args.append("--model-attributes")
         command_args.append(args.model_attributes)
@@ -470,6 +466,14 @@ def add_model_options(command_args, args):
             command_args.append(str(args.sample_rate))
         if args.replacement:
             command_args.append("--replacement")
+        if not args.ensemble_sample_replacement:
+            command_args.append("--ensemble-sample-no-replacement")
+        if args.ensemble_sample_seed:
+            command_args.append("--ensemble-sample-seed")
+            command_args.append(args.ensemble_sample_seed)
+        if args.ensemble_sample_rate < 1:
+            command_args.append("--ensemble-sample-rate")
+            command_args.append(str(args.ensemble_sample_rate))
         if args.randomize:
             command_args.append("--randomize")
         if args.ensemble_attributes:
@@ -478,7 +482,7 @@ def add_model_options(command_args, args):
     return command_args
 
 
-def create_kfold_evaluations(datasets_file, args, common_options,
+def create_kfold_evaluations(datasets_file, args, command_obj,
                              resume=False, counter=0):
     """ Create k-fold cross-validation from a datasets file
 
@@ -497,15 +501,21 @@ def create_kfold_evaluations(datasets_file, args, common_options,
     command = COMMANDS["create_cv"] % (datasets_file, output_dir, name,
                                        model_dataset)
     command_args = command.split()
+
     if model_fields:
         command_args.append("--model-fields")
         command_args.append(model_fields)
     command_args.append("--objective")
     command_args.append(args.objective_field)
     command_args = add_model_options(command_args, args)
-    common_options_list = u.get_options_list(args, common_options,
+    """
+    common_options_list = u.get_options_list(args, command_obj.common_options,
                                              prioritary=command_args)
     command_args.extend(common_options_list)
+    """
+    command_obj.propagate(command_args, exclude=["--dataset",
+                                                 "--datasets",
+                                                 "--dataset-file"])
     command = rebuild_command(command_args)
     if resume:
         next_command = subcommand_list.pop()
@@ -552,7 +562,7 @@ def expand_state(parent):
     return children
 
 
-def best_first_search(datasets_file, api, args, common_options,
+def best_first_search(datasets_file, api, args, command_obj,
                       staleness=None, penalty=None, objective_name=None,
                       resume=False):
     """Selecting the fields to be used in the model construction
@@ -596,6 +606,7 @@ def best_first_search(datasets_file, api, args, common_options,
     field_ids = [field_id for field_id in fields.preferred_fields()
                  if field_id != objective_id and
                  not field_id in excluded_ids]
+    field_ids.sort()
     # headers are extended with a column per field
     fields_names = [fields.field_name(field_id) for field_id in field_ids]
     features_header.extend(fields_names)
@@ -609,9 +620,10 @@ def best_first_search(datasets_file, api, args, common_options,
     while best_unchanged_count < staleness and open_list:
         loop_counter += 1
         features_set = find_max_state(open_list)
-        state, score, metric_value, folder_counter = features_set
+        state, score, metric_value, _ = features_set
         if loop_counter > 1:
-            csv_results = [loop_counter - 1, [int(in_set) for in_set in state],
+            csv_results = [loop_counter - 1, \
+                [int(in_set) for in_set in state], \
                 score, metric_value, best_score]
             csv_results.extend([int(in_set) for in_set in state])
             features_writer.writerow(csv_results)
@@ -661,7 +673,7 @@ def best_first_search(datasets_file, api, args, common_options,
                  metric_value,
                  metric,
                  resume) = kfold_evaluate(datasets_file,
-                                          args, counter, common_options,
+                                          args, counter, command_obj,
                                           penalty=penalty, resume=resume,
                                           metric=metric)
                 open_list.append((child, score, metric_value, counter))
@@ -704,7 +716,7 @@ def extract_evaluation_info(evaluation, category):
     return evaluation
 
 
-def kfold_evaluate(datasets_file, args, counter, common_options,
+def kfold_evaluate(datasets_file, args, counter, command_obj,
                    penalty=DEFAULT_PENALTY,
                    metric=ACCURACY, resume=False):
     """Scoring k-fold cross-validation using the given feature subset
@@ -714,7 +726,7 @@ def kfold_evaluate(datasets_file, args, counter, common_options,
     args.output_dir = os.path.normpath(os.path.join(u.check_dir(datasets_file),
                                                     "kfold"))
     evaluation, resume = create_kfold_evaluations(datasets_file, args,
-                                                  common_options,
+                                                  command_obj,
                                                   resume=resume,
                                                   counter=counter)
 
@@ -736,7 +748,7 @@ def kfold_evaluate(datasets_file, args, counter, common_options,
             metric_literal, resume)
 
 
-def best_node_threshold(datasets_file, args, common_options,
+def best_node_threshold(datasets_file, args, command_obj,
                         staleness=None, penalty=None,
                         resume=False):
     """Selecting the node_limit to be used in the model construction
@@ -771,7 +783,7 @@ def best_node_threshold(datasets_file, args, common_options,
          metric_value,
          metric,
          resume) = node_threshold_evaluate(datasets_file, args,
-                                           node_threshold, common_options,
+                                           node_threshold, command_obj,
                                            penalty=penalty, resume=resume,
                                            metric=metric)
         nodes_writer.writerow([
@@ -797,7 +809,7 @@ def best_node_threshold(datasets_file, args, common_options,
             best_unchanged_count += 1
         node_threshold += args.nodes_step
     if args.predictions_csv:
-        resume = create_prediction_dataset(output_dir,
+        resume = create_prediction_dataset(args.output_dir,
                                            "node_th%s" % best_counter,
                                            args, resume)
     message = ('The best node threshold is: %s \n'
@@ -814,14 +826,14 @@ def best_node_threshold(datasets_file, args, common_options,
 
 
 def node_threshold_evaluate(datasets_file, args, node_threshold,
-                            common_options, penalty=DEFAULT_NODES_PENALTY,
+                            command_obj, penalty=DEFAULT_NODES_PENALTY,
                             metric=ACCURACY, resume=False):
     """Scoring node_threshold created models
 
     """
     # create evaluation with input_fields
     evaluation, resume = create_node_th_evaluations(
-        datasets_file, args, common_options, resume=resume,
+        datasets_file, args, command_obj, resume=resume,
         node_threshold=node_threshold)
 
     evaluation = extract_evaluation_info(
@@ -841,7 +853,7 @@ def node_threshold_evaluate(datasets_file, args, node_threshold,
             metric_literal, resume)
 
 
-def create_node_th_evaluations(datasets_file, args, common_options,
+def create_node_th_evaluations(datasets_file, args, command_obj,
                                resume=False,
                                node_threshold=DEFAULT_MIN_NODES):
     """ Create node_threshold evaluations
@@ -854,12 +866,9 @@ def create_node_th_evaluations(datasets_file, args, common_options,
     command = COMMANDS["node_threshold"] % (
         datasets_file, node_threshold, output_dir)
     command_args = command.split()
-    common_options_list = u.get_options_list(args, common_options,
-                                             prioritary=command_args)
-    command_args.extend(common_options_list)
-    command_args.append("--objective")
-    command_args.append(args.objective_field)
-    command_args = add_model_options(command_args, args)
+    command_obj.propagate(command_args, exclude=["--dataset",
+                                                 "--datasets",
+                                                 "--dataset-file"])
     command = rebuild_command(command_args)
     if resume:
         next_command = subcommand_list.pop()
@@ -892,7 +901,7 @@ def default_candidates_limits(args, fields):
     args.max_candidates = len(fields.preferred_fields().keys())
 
 
-def create_candidates_analysis(args, api, common_options, resume=False):
+def create_candidates_analysis(args, api, command_obj, resume=False):
     """Analyzes the model performance as a function of the number of
        random candidates.
 
@@ -902,7 +911,7 @@ def create_candidates_analysis(args, api, common_options, resume=False):
     if resume:
         retrieve_subcommands()
     datasets_file, objective_name, resume = create_kfold_datasets_file(
-        args, api, common_options, resume=resume)
+        args, api, command_obj, resume=resume)
     args.objective_field = objective_name
     if args.number_of_models == 1:
         args.number_of_models = DEFAULT_NUMBER_OF_MODELS
@@ -910,7 +919,7 @@ def create_candidates_analysis(args, api, common_options, resume=False):
     u.log_message(message, log_file=session_file,
                   console=args.verbosity)
     random_candidates = best_candidates_number(
-        datasets_file, args, common_options,
+        datasets_file, args, command_obj,
         penalty=args.penalty,
         resume=resume)
     # showing the instruction to create the complete model with the
@@ -930,7 +939,7 @@ def create_candidates_analysis(args, api, common_options, resume=False):
     u.log_message(message, log_file=session_file, console=1)
 
 
-def best_candidates_number(datasets_file, args, common_options,
+def best_candidates_number(datasets_file, args, command_obj,
                            penalty=None,
                            resume=False):
     """Selecting the best number of random candidates
@@ -962,7 +971,7 @@ def best_candidates_number(datasets_file, args, common_options,
          metric_value,
          metric,
          resume) = candidates_evaluate(datasets_file, args,
-                                       random_candidates, common_options,
+                                       random_candidates, command_obj,
                                        penalty=penalty, resume=resume,
                                        metric=metric)
         candidates_writer.writerow([
@@ -987,7 +996,7 @@ def best_candidates_number(datasets_file, args, common_options,
                           console=args.verbosity)
         random_candidates += DEFAULT_CANDIDATES_STEP
     if args.predictions_csv:
-        resume = create_prediction_dataset(output_dir,
+        resume = create_prediction_dataset(args.output_dir,
                                            "random%s" % best_counter,
                                            args, resume)
     message = ('The best random candidates number is: %s \n'
@@ -1004,14 +1013,14 @@ def best_candidates_number(datasets_file, args, common_options,
 
 
 def candidates_evaluate(datasets_file, args, random_candidates,
-                        common_options, penalty=DEFAULT_CANDIDATES_PENALTY,
+                        command_obj, penalty=DEFAULT_CANDIDATES_PENALTY,
                         metric=ACCURACY, resume=False):
     """Scoring random candidates ensembles
 
     """
     # create evaluation with input_fields
     evaluation, resume = create_candidates_evaluations(
-        datasets_file, args, common_options, resume=resume,
+        datasets_file, args, command_obj, resume=resume,
         random_candidates=random_candidates)
 
     evaluation = extract_evaluation_info(
@@ -1031,7 +1040,7 @@ def candidates_evaluate(datasets_file, args, random_candidates,
             metric_literal, resume)
 
 
-def create_candidates_evaluations(datasets_file, args, common_options,
+def create_candidates_evaluations(datasets_file, args, command_obj,
                                   resume=False,
                                   random_candidates=DEFAULT_MIN_CANDIDATES):
     """ Create random candidates ensembles evaluations
@@ -1044,12 +1053,18 @@ def create_candidates_evaluations(datasets_file, args, common_options,
     command = COMMANDS["random_candidates"] % (
         datasets_file, random_candidates, output_dir)
     command_args = command.split()
-    common_options_list = u.get_options_list(args, common_options,
+    """
+    common_options_list = u.get_options_list(args, command_obj.common_options,
                                              prioritary=command_args)
     command_args.extend(common_options_list)
+    """
     command_args.append("--objective")
     command_args.append(args.objective_field)
     command_args = add_model_options(command_args, args)
+
+    command_obj.propagate(command_args, exclude=["--dataset",
+                                                 "--datasets",
+                                                 "--dataset-file"])
     command = rebuild_command(command_args)
     if resume:
         next_command = subcommand_list.pop()
