@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,13 +22,13 @@ import sys
 
 import bigml.api
 
-import bigmler.utils as u
-import bigmler.checkpoint as c
-
-
 from bigml.anomaly import Anomaly
 from bigml.io import UnicodeWriter
 
+
+import bigmler.utils as u
+import bigmler.checkpoint as c
+
 from bigmler.tst_reader import TstReader as TestReader
 from bigmler.resources import NORMAL_FORMAT, FULL_FORMAT
 from bigmler.resources import create_batch_anomaly_score
@@ -124,13 +124,12 @@ def local_anomaly_score(anomalies, test_reader, output, args,
 
     """
     # Only one anomaly detector at present
-    local_anomaly = Anomaly(anomalies[0])
-    test_set_header = test_reader.has_headers()
+    local_anomaly = Anomaly(anomalies[0], api=args.retrieve_api_)
     for input_data in test_reader:
         input_data_dict = test_reader.dict(input_data, filtering=False)
         try:
             anomaly_score_info = {'score': local_anomaly.anomaly_score(
-                input_data_dict, by_name=test_set_header)}
+                input_data_dict)}
         except Exception:
             anomaly_score_info = {'score': NO_ANOMALY_SCORE}
         write_anomaly_score(anomaly_score_info['score'], output,
