@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2016 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -89,7 +89,6 @@ FLAGS = {
         {'flag': 'test_split', 'type': 'float'},
         {'flag': 'ensemble', 'type': 'string'},
         {'flag': 'ensemble_file', 'type': 'string'},
-        {'flag': 'tlp', 'type': 'int'},
         {'flag': 'prediction_info', 'type': 'string'},
         {'flag': 'max_parallel_evaluations', 'type': 'int'},
         {'flag': 'test_separator', 'type': 'string'},
@@ -154,7 +153,21 @@ FLAGS = {
         {'flag': 'random_candidates', 'type': 'int'},
         {'flag': 'status', 'type': 'string'},
         {'flag': 'export_fields', 'type': 'string'},
-        {'flag': 'import_fields', 'type': 'string'}],
+        {'flag': 'import_fields', 'type': 'string'},
+        {'flag': 'only_execution', 'type': 'boolean'},
+        {'flag': 'ensemble_sample_seed', 'type': 'string'},
+        {'flag': 'ensemble_sample_rate', 'type': 'float'},
+        {'flag': 'ensemble_sample_replacement', 'type': 'boolean'},
+        {'flag': 'boosting', 'type': 'boolean'},
+        {'flag': 'iterations', 'type': 'int'},
+        {'flag': 'early_holdout', 'type': 'float'},
+        {'flag': 'early_out_of_bag', 'type': 'boolean'},
+        {'flag': 'learning_rate', 'type': 'float'},
+        {'flag': 'operating_point', 'type': 'string'},
+        {'flag': 'step_out_of_bag', 'type': 'boolean'},
+        {'flag': 'org_project', 'type': 'string'},
+        {'flag': 'split_field', 'type': 'string'},
+        {'flag': 'float_field', 'type': 'string'}],
     'BigMLer analyze': [
         {'flag': 'k-fold', 'type': 'int'},
         {'flag': 'cv', 'type': 'boolean'},
@@ -239,11 +252,129 @@ FLAGS = {
         {'flag': 'logistic_regressions', 'type': 'string'},
         {'flag': 'no_logistic_regression', 'type': 'boolean'},
         {'flag': 'logistic_regression_attributes', 'type': 'string'},
-        {'flag': 'bias', 'type': 'float'},
+        {'flag': 'bias', 'type': 'boolean'},
+        {'flag': 'balance_fields', 'type': 'boolean'},
         {'flag': 'lr_c', 'type': 'float'},
         {'flag': 'eps', 'type': 'float'},
+        {'flag': 'field_codings', 'type': 'string'},
         {'flag': 'missing_numerics', 'type': 'boolean'},
-        {'flag': 'normalize', 'type': 'boolean'}]}
+        {'flag': 'normalize', 'type': 'boolean'}],
+    'BigMLer linear regression': [
+        {'flag': 'linear_fields', 'type': 'string'},
+        {'flag': 'linear_regression', 'type': 'string'},
+        {'flag': 'linear_regression_file', 'type': 'string'},
+        {'flag': 'linear_regressions', 'type': 'string'},
+        {'flag': 'no_linear_regression', 'type': 'boolean'},
+        {'flag': 'linear_regression_attributes', 'type': 'string'},
+        {'flag': 'bias', 'type': 'boolean'},
+        {'flag': 'field_codings', 'type': 'string'}],
+    'BigMLer topic model': [
+        {'flag': 'topic_fields', 'type': 'string'},
+        {'flag': 'topic_model', 'type': 'string'},
+        {'flag': 'topic_model_file', 'type': 'string'},
+        {'flag': 'topic_models', 'type': 'string'},
+        {'flag': 'no_topic_model', 'type': 'boolean'},
+        {'flag': 'topic_model_attributes', 'type': 'string'},
+        {'flag': 'bigrams', 'type': 'boolean'},
+        {'flag': 'case_sensitive', 'type': 'boolean'},
+        {'flag': 'excluded_terms', 'type': 'string'},
+        {'flag': 'number_of_topics', 'type': 'int'},
+        {'flag': 'term_limit', 'type': 'int'},
+        {'flag': 'top_n_terms', 'type': 'int'},
+        {'flag': 'topic_model_attributes', 'type': 'string'},
+        {'flag': 'use_stopwords', 'type': 'boolean'}],
+    'BigMLer time-series': [
+        {'flag': 'objectives', 'type': 'string'},
+        {'flag': 'time_series', 'type': 'string'},
+        {'flag': 'time_series_file', 'type': 'string'},
+        {'flag': 'time_series_set', 'type': 'string'},
+        {'flag': 'no_time_series', 'type': 'boolean'},
+        {'flag': 'time_series_attributes', 'type': 'string'},
+        {'flag': 'all_numeric_objectives', 'type': 'boolean'},
+        {'flag': 'damped_trend', 'type': 'boolean'},
+        {'flag': 'default_numeric_value', 'type': 'string'},
+        {'flag': 'error', 'type': 'int'},
+        {'flag': 'period', 'type': 'int'},
+        {'flag': 'field_parameters', 'type': 'string'},
+        {'flag': 'time_series_attributes', 'type': 'string'},
+        {'flag': 'range', 'type': 'string'},
+        {'flag': 'seasonality', 'type': 'int'},
+        {'flag': 'trend', 'type': 'int'},
+        {'flag': 'time_start', 'type': 'int'},
+        {'flag': 'time_end', 'type': 'int'},
+        {'flag': 'time_interval', 'type': 'int'},
+        {'flag': 'time_interval_unit', 'type': 'string'},
+        {'flag': 'error', 'type': 'int'},
+        {'flag': 'interval', 'type': 'boolean'}],
+    'BigMLer deepnet': [
+        {'flag': 'deepnet_fields', 'type': 'string'},
+        {'flag': 'deepnet', 'type': 'string'},
+        {'flag': 'deepnet_file', 'type': 'string'},
+        {'flag': 'deepnets', 'type': 'string'},
+        {'flag': 'no_deepnet', 'type': 'boolean'},
+        {'flag': 'deepnet_attributes', 'type': 'string'},
+        {'flag': 'batch_normalization', 'type': 'boolean'},
+        {'flag': 'default_numeric_value', 'type': 'string'},
+        {'flag': 'dropout_rate', 'type': 'float'},
+        {'flag': 'hidden_layers', 'type': 'string'},
+        {'flag': 'learn_residuals', 'type': 'boolean'},
+        {'flag': 'learning_rate', 'type': 'float'},
+        {'flag': 'max_iterations', 'type': 'int'},
+        {'flag': 'max_training_time', 'type': 'int'},
+        {'flag': 'number_of_hidden_layers', 'type': 'int'},
+        {'flag': 'number_of_model_candidates', 'type': 'int'},
+        {'flag': 'search', 'type': 'boolean'},
+        {'flag': 'suggest_structure', 'type': 'boolean'},
+        {'flag': 'missing_numerics', 'type': 'boolean'},
+        {'flag': 'no_missing_numerics', 'type': 'boolean'},
+        {'flag': 'tree_embedding', 'type': 'boolean'},
+        {'flag': 'no_balance_fields', 'type': 'boolean'},
+        {'flag': 'deepnet_attributes', 'type': 'string'}],
+    'BigMLer execute': [
+        {'flag': 'script', 'type': 'string'},
+        {'flag': 'library', 'type': 'string'},
+        {'flag': 'execution', 'type': 'string'},
+        {'flag': 'code_file', 'type': 'string'},
+        {'flag': 'embedded_imports', 'type': 'string'},
+        {'flag': 'output', 'type': 'string'},
+        {'flag': 'imports', 'type': 'string'},
+        {'flag': 'inputs', 'type': 'string'},
+        {'flag': 'declare_inputs', 'type': 'string'},
+        {'flag': 'declare_outputs', 'type': 'string'},
+        {'flag': 'input_maps', 'type': 'string'},
+        {'flag': 'outputs', 'type': 'string'},
+        {'flag': 'creation_defaults', 'type': 'string'},
+        {'flag': 'no_execute', 'type': 'boolean'}],
+    'BigMLer whizzml': [
+        {'flag': 'package_dir', 'type': 'string'},
+        {'flag': 'embed_libs', 'type': 'boolean'}],
+    'BigMLer export': [
+        {'flag': 'language', 'type': 'string'}],
+    'BigMLer retrain': [
+        {'flag': 'model_type', 'type': 'string'}],
+    'BigMLer PCA': [
+        {'flag': 'pca_fields', 'type': 'string'},
+        {'flag': 'pca', 'type': 'string'},
+        {'flag': 'pca_file', 'type': 'string'},
+        {'flag': 'pcas', 'type': 'string'},
+        {'flag': 'no_pca', 'type': 'boolean'},
+        {'flag': 'max_components', 'type': 'int'},
+        {'flag': 'variance_threshold', 'type': 'float'},
+        {'flag': 'pca_attributes', 'type': 'string'}],
+    'BigMLer Fusion': [
+        {'flag': 'fusion_models', 'type': 'string'},
+        {'flag': 'fusion_models_file', 'type': 'string'},
+        {'flag': 'fusion_file', 'type': 'string'},
+        {'flag': 'fusions', 'type': 'string'},
+        {'flag': 'fusion', 'type': 'string'},
+        {'flag': 'fusion_attributes', 'type': 'string'}],
+    'BigMLer dataset': [
+        {'flag': 'file', 'type': 'string'},
+        {'flag': 'merge', 'type': 'boolean'},
+        {'flag': 'juxtapose', 'type': 'boolean'},
+        {'flag': 'sql_query', 'type': 'string'},
+        {'flag': 'json_query', 'type': 'string'},
+        {'flag': 'sql_output_fields', 'type': 'string'}]}
 
 
 def get_user_defaults(defaults_file=DEFAULTS_FILE):
