@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2016 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -46,7 +46,7 @@ def evaluate(models_or_ensembles, datasets, api, args, resume,
         dataset_fields, api, args, resume,
         session_file=session_file, path=path, log=log,
         labels=labels, all_labels=all_labels, objective_field=objective_field)
-    if args.multi_label:
+    if hasattr(args, 'multi_label') and args.multi_label:
         file_labels = [slugify(name) for name in
                        u.objective_field_names(models_or_ensembles, api)]
     for index in range(0, len(evaluations)):
@@ -59,7 +59,7 @@ def evaluate(models_or_ensembles, datasets, api, args, resume,
                                              args, api=api, path=path,
                                              session_file=session_file)
         file_name = output
-        if args.multi_label:
+        if hasattr(args, 'multi_label') and args.multi_label:
             suffix = file_labels[index]
             file_name += "_%s" % suffix
             evaluation_files.append("%s.json" % file_name)
@@ -68,7 +68,8 @@ def evaluate(models_or_ensembles, datasets, api, args, resume,
             file_name += "_%s" % suffix
             evaluation_files.append("%s.json" % file_name)
         r.save_evaluation(evaluation, file_name, api)
-    if args.multi_label or args.test_datasets or args.dataset_off:
+    if (hasattr(args, 'multi_label') and args.multi_label) or \
+            args.test_datasets or args.dataset_off:
         mean_evaluation = average_evaluations(evaluation_files)
         r.save_evaluation(mean_evaluation, output, api)
     return resume
@@ -122,7 +123,7 @@ def evaluations_process(models_or_ensembles, datasets,
             u.log_message(message, log_file=session_file,
                           console=args.verbosity)
     if not resume:
-        if args.multi_label:
+        if hasattr(args, 'multi_label') and args.multi_label:
             evaluation_args = r.set_label_evaluation_args(
                 args, labels, all_labels,
                 number_of_evaluations, fields, dataset_fields,
@@ -231,8 +232,8 @@ def add_matrices(matrix_a, matrix_b):
     """Add two n x n matrices
 
     """
-    return map(lambda i: map(lambda x, y: x + y, matrix_a[i], matrix_b[i]),
-               xrange(len(matrix_a)))
+    return [[x + y  for x, y in zip(matrix_a[i], matrix_b[i])]
+            for i in xrange(len(matrix_a))]
 
 
 def avg_class_statistics(total, component, number_of_evaluations):
@@ -240,7 +241,15 @@ def avg_class_statistics(total, component, number_of_evaluations):
 
     """
     special_keys = ['class_name', 'present_in_test_data', 'occurrences']
+    remove_keys = ['ks_statistic', 'max_phi',
+                   'per_threshold_confusion_matrices',
+                   'roc_curve', 'pr_curve', 'negative_cdf', 'max_phi',
+                   'lift_curve', 'ks_statistic', 'gain_curve']
+
     for class_info in component:
+        for key in remove_keys:
+            if key in class_info:
+                del class_info[key]
         class_name = class_info['class_name']
         found = False
         for total_class_info in total:
@@ -253,28 +262,35 @@ def avg_class_statistics(total, component, number_of_evaluations):
                     total_class_info['occurrences'] -= 1
                     occurrences = float(total_class_info['occurrences'])
                     for key in total_class_info:
-                        # renormalizing previous average count
-                        if not (isinstance(key, tuple) or key in special_keys):
-                            total_class_info[key] *= ((occurrences + 1) /
-                                                      occurrences)
+                        try:
+                            # renormalizing previous average count
+                            if not (isinstance(key, tuple) or
+                                    key in special_keys):
+                                total_class_info[key] *= ((occurrences + 1) /
+                                                          occurrences)
+                        except (ValueError, TypeError):
+                            pass
                 if not total_class_info['present_in_test_data']:
                     total_class_info['present_in_test_data'] = flag
                 occurrences = float(total_class_info['occurrences'])
                 for key in class_info:
-                    if not (isinstance(key, tuple) or key in special_keys):
-                        new_key = (key if key.startswith("average_")
-                                   else ("average_%s" % key))
-                        if new_key in total_class_info:
-                            total_class_info[new_key] += (class_info[key] /
-                                                          occurrences)
-                        else:
-                            total_class_info[new_key] = (class_info[key] /
-                                                         occurrences)
-                        sd_key = "%s_standard_deviation" % key
-                        if not (sd_key, new_key) in total_class_info:
-                            total_class_info[(sd_key, new_key)] = []
-                        total_class_info[
-                            (sd_key, new_key)].append(class_info[key])
+                    try:
+                        if not (isinstance(key, tuple) or key in special_keys):
+                            new_key = (key if key.startswith("average_")
+                                       else ("average_%s" % key))
+                            if new_key in total_class_info:
+                                total_class_info[new_key] += (class_info[key] /
+                                                              occurrences)
+                            else:
+                                total_class_info[new_key] = (class_info[key] /
+                                                             occurrences)
+                            sd_key = "%s_standard_deviation" % key
+                            if not (sd_key, new_key) in total_class_info:
+                                total_class_info[(sd_key, new_key)] = []
+                            total_class_info[
+                                (sd_key, new_key)].append(class_info[key])
+                    except (ValueError, TypeError):
+                        pass
                 break
         if not found:
             flag = class_info['present_in_test_data']
@@ -283,22 +299,27 @@ def avg_class_statistics(total, component, number_of_evaluations):
                 class_info['occurrences'] -= 1
             keys = class_info.keys()
             for key in keys:
-                if not key in special_keys:
-                    sd_key = "%s_standard_deviation" % key
-                    if not key.startswith("average_"):
-                        new_key = "average_%s" % key
-                        class_info[new_key] = (float(class_info[key]) /
+                try:
+                    if not key in special_keys:
+                        sd_key = "%s_standard_deviation" % key
+                        if not key.startswith("average_"):
+                            new_key = "average_%s" % key
+                            class_info[new_key] = (float(class_info[key]) /
+                                                   class_info['occurrences'])
+                            if not (sd_key, new_key) in class_info:
+                                class_info[(sd_key, new_key)] = []
+                            class_info[(sd_key, new_key)].append( \
+                                class_info[key])
+                            del class_info[key]
+                        else:
+                            new_key = key
+                            class_info[key] = (float(class_info[key]) /
                                                class_info['occurrences'])
-                        if not (sd_key, new_key) in class_info:
-                            class_info[(sd_key, new_key)] = []
-                        class_info[(sd_key, new_key)].append(class_info[key])
-                        del class_info[key]
-                    else:
-                        new_key = key
-                        class_info[key] = (float(class_info[key]) /
-                                           class_info['occurrences'])
-                        if not (sd_key, new_key) in class_info:
-                            class_info[(sd_key, new_key)] = []
-                        class_info[(sd_key, new_key)].append(class_info[key])
+                            if not (sd_key, new_key) in class_info:
+                                class_info[(sd_key, new_key)] = []
+                            class_info[(sd_key, new_key)].append( \
+                                class_info[key])
+                except (ValueError, TypeError):
+                    pass
             total.append(class_info)
     return total
