@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,7 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""BigMLer - cluster subcommand processing dispatching
+"""BigMLer - logistic regression subcommand processing dispatching
 
 """
 from __future__ import absolute_import
@@ -33,10 +33,11 @@ import bigmler.processing.sources as ps
 import bigmler.processing.datasets as pd
 
 from bigmler.defaults import DEFAULTS_FILE
-from bigmler.lrprediction import lr_prediction, remote_lr_prediction
+from bigmler.sl_prediction import prediction, remote_prediction
 from bigmler.reports import clear_reports, upload_reports
-from bigmler.command import get_stored_command
-from bigmler.dispatcher import (SESSIONS_LOG, command_handling,
+from bigmler.command import get_context
+from bigmler.evaluation import evaluate
+from bigmler.dispatcher import (SESSIONS_LOG,
                                 clear_log_files, get_test_dataset,
                                 get_objective_id)
 
@@ -46,6 +47,13 @@ LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
 MINIMUM_MODEL = "full=false"
 DEFAULT_OUTPUT = u"predictions.csv"
 
+SETTINGS = {
+    "command_log": COMMAND_LOG,
+    "sessions_log": SESSIONS_LOG,
+    "dirs_log": DIRS_LOG,
+    "default_output": DEFAULT_OUTPUT,
+    "defaults_file": DEFAULTS_FILE}
+
 
 def logistic_regression_dispatcher(args=sys.argv[1:]):
     """Parses command line and calls the different processing functions
@@ -56,51 +64,18 @@ def logistic_regression_dispatcher(args=sys.argv[1:]):
     if "--clear-logs" in args:
         clear_log_files(LOG_FILES)
 
-    command = command_handling(args, COMMAND_LOG)
+    settings = {}
+    settings.update(SETTINGS)
+    if '--evaluate' in args:
+        settings.update({"default_output": "evaluation"})
 
-    # Parses command line arguments.
-    command_args = a.parse_and_check(command)
-    default_output = ('evaluation' if command_args.evaluate
-                      else 'predictions.csv')
-    resume = command_args.resume
-    if command_args.resume:
-        command_args, session_file, output_dir = get_stored_command(
-            args, command_args.debug, command_log=COMMAND_LOG,
-            dirs_log=DIRS_LOG, sessions_log=SESSIONS_LOG)
-        default_output = ('evaluation' if command_args.evaluate
-                          else 'predictions.csv')
-        if command_args.predictions is None:
-            command_args.predictions = os.path.join(output_dir,
-                                                    default_output)
-    else:
-        if command_args.output_dir is None:
-            command_args.output_dir = a.NOW
-        if command_args.predictions is None:
-            command_args.predictions = os.path.join(command_args.output_dir,
-                                                    default_output)
-        if len(os.path.dirname(command_args.predictions).strip()) == 0:
-            command_args.predictions = os.path.join(command_args.output_dir,
-                                                    command_args.predictions)
-        directory = u.check_dir(command_args.predictions)
-        session_file = os.path.join(directory, SESSIONS_LOG)
-        u.log_message(command.command + "\n", log_file=session_file)
-        try:
-            shutil.copy(DEFAULTS_FILE, os.path.join(directory, DEFAULTS_FILE))
-        except IOError:
-            pass
-        u.sys_log_message(u"%s\n" % os.path.abspath(directory),
-                          log_file=DIRS_LOG)
-
-    # Creates the corresponding api instance
-    api = a.get_api_instance(command_args, u.check_dir(session_file))
+    command_args, command, api, session_file, resume = get_context(args,
+                                                                   settings)
 
     # Selects the action to perform
     if (a.has_train(command_args) or a.has_test(command_args)
             or command_args.export_fields):
-        output_args = a.get_output_args(api, command_args, resume)
-        a.transform_args(command_args, command.flags, api,
-                         command.user_defaults)
-        compute_output(**output_args)
+        compute_output(api, command_args)
     u.log_message("_" * 80 + "\n", log_file=session_file)
 
 
@@ -216,7 +191,7 @@ def compute_output(api, args):
             logistic_regression, csv_properties, args)
 
     if fields and args.export_fields:
-       fields.summary_csv(os.path.join(path, args.export_fields))
+        fields.summary_csv(os.path.join(path, args.export_fields))
 
     # If predicting
     if logistic_regressions and (a.has_test(args) or \
@@ -256,14 +231,14 @@ def compute_output(api, args):
                 args, fields=fields,
                 dataset_fields=test_fields)
 
-            remote_lr_prediction(logistic_regression, test_dataset, \
+            remote_prediction(logistic_regression, test_dataset, \
                 batch_prediction_args, args, \
                 api, resume, prediction_file=output, \
                 session_file=session_file, path=path, log=log)
 
         else:
-            lr_prediction(logistic_regressions, fields, args,
-                          session_file=session_file)
+            prediction(logistic_regressions, fields, args,
+                       session_file=session_file)
 
     # If evaluate flag is on, create remote evaluation and save results in
     # json and human-readable format.
@@ -276,13 +251,34 @@ def compute_output(api, args):
             args.test_dataset_ids = datasets
         if args.test_dataset_ids and args.dataset_off:
             # Evaluate the models with the corresponding test datasets.
+            test_dataset_id = bigml.api.get_dataset_id( \
+                args.test_dataset_ids[0])
+            test_dataset = api.check_resource(test_dataset_id)
+            csv_properties.update(objective_field=None,
+                                  objective_field_present=False)
+            test_fields = pd.get_fields_structure(test_dataset,
+                                                  csv_properties)
             resume = evaluate(logistic_regressions, args.test_dataset_ids, api,
+                              args, resume,
+                              fields=fields, dataset_fields=test_fields,
+                              session_file=session_file, path=path,
+                              log=log,
+                              objective_field=args.objective_field)
+        else:
+            dataset = datasets[0]
+            if args.test_split > 0 or args.has_test_datasets_:
+                dataset = test_dataset
+            dataset = u.check_resource(dataset, api=api,
+                                       query_string=r.ALL_FIELDS_QS)
+            dataset_fields = pd.get_fields_structure(dataset, None)
+            resume = evaluate(logistic_regressions, [dataset], api,
                               args, resume,
                               fields=fields, dataset_fields=dataset_fields,
                               session_file=session_file, path=path,
-                              log=log, labels=labels, all_labels=all_labels,
+                              log=log,
                               objective_field=args.objective_field)
 
+
     u.print_generated_files(path, log_file=session_file,
                             verbosity=args.verbosity)
     if args.reports:
