@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,6 +19,8 @@
 """
 from __future__ import absolute_import
 
+import os
+
 import bigmler.processing.sources as ps
 import bigmler.processing.datasets as pd
 import bigmler.utils as u
@@ -43,7 +45,7 @@ def get_source_info(api, args, resume,
             session_file=session_file, path=path, log=log)
 
     if fields and args.export_fields:
-       fields.summary_csv(os.path.join(path, args.export_fields))
+        fields.summary_csv(os.path.join(path, args.export_fields))
 
     return source, resume, csv_properties, fields
 
@@ -82,10 +84,18 @@ def get_dataset_info(api, args, resume, source,
     # If test_split is used, split the dataset in a training and a test dataset
     # according to the given split
     if args.test_split > 0:
-        dataset, test_dataset, resume = pd.split_processing(
-            dataset, api, args, resume,
-            session_file=session_file, path=path, log=log)
-        datasets[0] = dataset
+        if args.subcommand == "time-series":
+            # use ranges
+            dataset, test_dataset, resume = pd.split_range_processing(
+                dataset, api, args, resume,
+                session_file=session_file, path=path, log=log)
+            datasets[0] = dataset
+        else:
+            # use sample_rate
+            dataset, test_dataset, resume = pd.split_processing(
+                dataset, api, args, resume,
+                session_file=session_file, path=path, log=log)
+            datasets[0] = dataset
 
     # If multi-dataset flag is on, generate a new dataset from the given
     # list of datasets
@@ -104,6 +114,6 @@ def get_dataset_info(api, args, resume, source,
         datasets[0] = dataset
 
     if fields and args.export_fields:
-       fields.summary_csv(os.path.join(path, args.export_fields))
+        fields.summary_csv(os.path.join(path, args.export_fields))
 
     return dataset, datasets, test_dataset, resume, csv_properties, fields
