@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,19 +22,19 @@ from __future__ import absolute_import
 import sys
 import os
 import datetime
+import json
 
 import bigml.api
 
-import bigmler.utils as u
-
-
 from bigml.multivote import COMBINATION_WEIGHTS, COMBINER_MAP
 from bigml.tree import LAST_PREDICTION, PROPORTIONAL
 
+import bigmler.utils as u
+
 from bigmler.resources import ADD_REMOVE_PREFIX
 from bigmler.prediction import FULL_FORMAT, COMBINATION, COMBINATION_LABEL
 from bigmler.train_reader import AGGREGATES
-from bigmler.utils import PYTHON3
+from bigmler.utils import PYTHON3, check_dir
 
 if PYTHON3:
     from io import StringIO
@@ -47,42 +47,68 @@ MISSING_STRATEGIES = {'last': LAST_PREDICTION, 'proportional': PROPORTIONAL}
 DEFAULT_DESCRIPTION = "Created using BigMLer"
 RESOURCE_TYPES = ["source", "dataset", "model", "ensemble", "batch_prediction",
                   "cluster", "centroid", "batch_centroid", "anomaly",
-                  "anomaly_score", "batch_anomaly_score", "association",
-                  "logistic_regression", "project"]
+                  "anomaly_score", "batch_anomaly_score", "project", "sample",
+                  "association", "logistic_regression", "deepnet", "script",
+                  "library", "execution", "evaluation", "topic_model",
+                  "linear_regression", "fusion", "prediction", "pca"]
+
+STORED_MODELS = ["model_file", "ensemble_file", "logistic_file",
+                 "cluster_file", "anomaly_file", "deepnet_file",
+                 "linear_file", "topic_model_file", "association_file",
+                 "pca_file"]
+
+def has_value(args, attrs):
+    """Returns a list of booleans checking whether each attribute is set and
+       his value
+
+    """
+    if isinstance(attrs, list):
+        return [hasattr(args, attr) and getattr(args, attr)
+                for attr in attrs]
+    elif isinstance(attrs, basestring):
+        return hasattr(args, attrs) and getattr(args, attrs)
+    return false
 
 
 def has_test(args):
     """Returns if some kind of test data is given in args.
 
     """
-    return (args.test_set or args.test_source or args.test_dataset or
-            args.test_stdin or args.test_datasets)
+    return any(has_value(args, ["test_set", "test_source", "test_dataset",
+                                "test_stdin", "test_datasets"]))
+
+def has_ts_test(args):
+    """Returns if some kind of test data for time series is given in args.
+
+    """
+    return any(has_value(args, ["test_set", "test_stdin", "forecast",
+                                "horizon"]))
 
 
 def has_train(args):
     """Returns if some kind of train data is given in args.
 
     """
-    return (args.training_set or args.source or args.dataset or
-            args.datasets or args.source_file or args.dataset_file or
-            args.train_stdin)
+    return any(has_value(args, ["training_set", "source", "dataset",
+                                "datasets", "source_file", "dataset_file",
+                                "train_stdin"]))
 
 
 def has_model(args):
     """Boolean that is set when any model option is used
 
     """
-    return (args.model or args.models or args.model_tag or args.model_file
-            or args.ensemble or args.ensembles or args.ensemble_tag
-            or args.ensemble_file)
+    return any(has_value(args, ["model", "models", "model_tag", "model_file",
+                                "ensemble", "ensembles", "ensemble_tag",
+                                "ensemble_file"]))
 
 
 def has_anomaly(args):
     """Boolean that is set when any anomaly option is used
 
     """
-    return (args.anomaly or args.anomalies or args.anomaly_tag or
-            args.anomaly_file)
+    return any(has_value(args, ["anomaly", "anomalies", "anomaly_tag",
+                                "anomaly_file"]))
 
 
 def non_compatible(args, option):
@@ -90,10 +116,10 @@ def non_compatible(args, option):
 
     """
     if option == '--cross-validation-rate':
-        return (args.test_set or args.evaluate or args.model or args.models or
-                args.model_tag or args.multi_label)
+        return any(has_value(args, ["test_set", "evaluate", "model",
+                                    "models", "model_tag", "multi_label"]))
     if option == '--max-categories':
-        return args.evaluate or args.test_split or args.remote
+        return any(has_value(args, ["evaluate", "test_split", "remote"]))
     return False
 
 
@@ -155,6 +181,9 @@ def parse_and_check(command):
     command_args = parser.parse_args(args)
     command_args.train_stdin = command.train_stdin
     command_args.test_stdin = command.test_stdin
+    command_args.subcommand = command.command.split(" ")[1]
+    if command_args.subcommand.startswith("-"):
+        command_args.subcommand = "main"
 
     # Checks options' compatibility
     try:
@@ -229,13 +258,36 @@ def get_api_instance(command_args, storage_path):
     api_command_args = {
         'username': command_args.username,
         'api_key': command_args.api_key,
-        'dev_mode': command_args.dev_mode,
         'debug': command_args.debug}
 
     if command_args.store:
         api_command_args.update({'storage': storage_path})
+    if hasattr(command_args, "org_project") and command_args.org_project:
+        api_command_args.update({'project': command_args.org_project})
+    if hasattr(command_args, "organization") and command_args.organization:
+        api_command_args.update({"organization": command_args.organization})
 
-    return bigml.api.BigML(**api_command_args)
+    command_args.api_ = bigml.api.BigML(**api_command_args)
+
+    # if locally stored models are used, local predicting objects should use
+    # this directory to look for the model information first. Otherwise,
+    # use the storage_path and if not set ./storage directory
+    retrieve_dir = None
+    for stored_model in STORED_MODELS:
+        if hasattr(command_args, stored_model) and \
+                getattr(command_args, stored_model) is not None:
+            retrieve_dir = check_dir(getattr(command_args, stored_model))
+            break
+    if retrieve_dir is None:
+        retrieve_dir = storage_path if command_args.store else './storage'
+
+    command_args.retrieve_api_ = bigml.api.BigML(**{ \
+        'username': command_args.username,
+        'api_key': command_args.api_key,
+        'debug': command_args.debug,
+        'storage': retrieve_dir})
+
+    return command_args.api_
 
 
 def get_output_args(api, command_args, resume):
@@ -269,6 +321,9 @@ def get_output_args(api, command_args, resume):
         pass
 
     command_args.resume_ = resume
+    command_args.predictions = command_args.output
+    command_args.projections = command_args.output
+
 
     # Reads description if provided.
     try:
@@ -390,6 +445,157 @@ def get_output_args(api, command_args, resume):
     except AttributeError:
         pass
 
+    # Parses linear regression input fields if provided.
+    try:
+        if command_args.linear_fields:
+            linear_fields_arg = [
+                field.strip() for field in command_args.linear_fields.split(
+                    command_args.args_separator)]
+            command_args.linear_fields_ = linear_fields_arg
+        else:
+            command_args.linear_fields_ = []
+    except AttributeError:
+        pass
+
+    # Parses deepnet input fields if provided.
+    try:
+        if command_args.deepnet_fields:
+            deepnet_fields_arg = [
+                field.strip() for field in command_args.deepnet_fields.split(
+                    command_args.args_separator)]
+            command_args.deepnet_fields_ = deepnet_fields_arg
+        else:
+            command_args.deepnet_fields_ = []
+    except AttributeError:
+        pass
+
+    # Parses topic model fields if provided.
+    try:
+        if command_args.topic_fields:
+            topic_fields_arg = [
+                field.strip() for field in command_args.topic_fields.split(
+                    command_args.args_separator)]
+            command_args.topic_model_fields_ = topic_fields_arg
+        else:
+            command_args.topic_model_fields_ = []
+    except AttributeError:
+        pass
+
+
+    # Parses pca fields if provided.
+    try:
+        if command_args.pca_fields:
+            pca_fields_arg = [
+                field.strip() for field in command_args.pca_fields.split(
+                    command_args.args_separator)]
+            command_args.pca_fields_ = pca_fields_arg
+        else:
+            command_args.pca_fields_ = []
+    except AttributeError:
+        pass
+
+    # Parses field_codings for deepnet
+    try:
+        if command_args.field_codings:
+            command_args.field_codings_ = u.read_json(
+                command_args.field_codings)
+        else:
+            command_args.field_codings_ = []
+    except AttributeError:
+        pass
+
+    # Parses imports for scripts and libraries.
+    try:
+        if command_args.imports:
+            imports_arg = [
+                field.strip() for field in command_args.imports.split(
+                    command_args.args_separator)]
+            command_args.imports_ = imports_arg
+        else:
+            command_args.imports_ = []
+    except AttributeError:
+        pass
+
+    # Parses objective fields for time-series.
+    try:
+        if command_args.objectives:
+            objective_fields_arg = [
+                field.strip() for field in command_args.objectives.split(
+                    command_args.args_separator)]
+            command_args.objective_fields_ = objective_fields_arg
+        else:
+            command_args.objective_fields_ = []
+    except AttributeError:
+        pass
+
+    # Parses range.
+    try:
+        if command_args.range:
+            range_arg = [
+                value.strip() for value in command_args.range.split(
+                    command_args.args_separator)]
+            command_args.range_ = range_arg
+        else:
+            command_args.range_ = []
+    except AttributeError:
+        pass
+
+    # Parses parameters for scripts.
+    try:
+        if command_args.declare_inputs:
+            command_args.parameters_ = u.read_json(command_args.declare_inputs)
+        else:
+            command_args.parameters_ = []
+    except AttributeError:
+        pass
+
+    # Parses creation_defaults for executions.
+    try:
+        if command_args.creation_defaults:
+            command_args.creation_defaults_ = u.read_json(
+                command_args.creation_defaults)
+        else:
+            command_args.creation_defaults_ = {}
+    except AttributeError:
+        pass
+
+    # Parses arguments for executions.
+    try:
+        if command_args.inputs:
+            command_args.arguments_ = u.read_json(command_args.inputs)
+        else:
+            command_args.arguments_ = []
+    except AttributeError:
+        pass
+
+    # Parses input maps for executions.
+    try:
+        if command_args.input_maps:
+            command_args.input_maps_ = u.read_json(command_args.input_maps)
+        else:
+            command_args.input_maps_ = []
+    except AttributeError:
+        pass
+
+    # Parses outputs for executions.
+    try:
+        if command_args.outputs:
+            command_args.outputs_ = u.read_json(command_args.outputs)
+        else:
+            command_args.outputs_ = []
+    except AttributeError:
+        pass
+
+    # Parses outputs for scripts.
+    try:
+        if command_args.declare_outputs:
+            command_args.declare_outputs_ = \
+                u.read_json(command_args.declare_outputs)
+        else:
+            command_args.declare_outputs_ = []
+    except AttributeError:
+        pass
+
     model_ids = []
     try:
         # Parses model/ids if provided.
@@ -463,7 +669,7 @@ def get_output_args(api, command_args, resume):
     except AttributeError:
         pass
 
-    # Retrieve cluster/ids if provided.
+    # Retrieve association/ids if provided.
     try:
         if command_args.association_tag:
             association_ids = (association_ids +
@@ -484,17 +690,164 @@ def get_output_args(api, command_args, resume):
     except AttributeError:
         pass
 
-    # Retrieve logisticregression/ids if provided.
+    # Retrieve logsticregression/ids if provided.
     try:
-        if command_args.logistic_tag:
-            logistic_regression_ids = (logistic_ids +
-                           u.list_ids(api.list_logistic_regressions,
-                                      "tags__in=%s" %
-                                      command_args.logistic_tag))
+        if command_args.logistic_regression_tag:
+            logistic_regression_ids = (logistic_regression_ids + \
+                u.list_ids(api.list_logistic_regressions,
+                           "tags__in=%s" %
+                           command_args.logistic_regression_tag))
         command_args.logistic_regression_ids_ = logistic_regression_ids
     except AttributeError:
         pass
 
+    linear_regression_ids = []
+    try:
+        # Parses linearregression/ids if provided.
+        if command_args.linear_regressions:
+            linear_regression_ids = u.read_resources( \
+                command_args.linear_regressions)
+        command_args.linear_regression_ids_ = linear_regression_ids
+    except AttributeError:
+        pass
+
+    # Retrieve linearregression/ids if provided.
+    try:
+        if command_args.linear_regression_tag:
+            linear_regression_ids = (linear_regression_ids + \
+                u.list_ids(api.list_linear_regressions,
+                           "tags__in=%s" %
+                           command_args.linear_regression_tag))
+        command_args.linear_regression_ids_ = linear_regression_ids
+    except AttributeError:
+        pass
+
+
+    deepnet_ids = []
+    try:
+        # Parses deepnet/ids if provided.
+        if command_args.deepnets:
+            deepnet_ids = u.read_resources( \
+                command_args.deepnets)
+        command_args.deepnet_ids_ = deepnet_ids
+    except AttributeError:
+        pass
+
+    # Retrieve deepnet/ids if provided.
+    try:
+        if command_args.deepnet_tag:
+            deepnet_regression_ids = (deepnet_ids + \
+                u.list_ids(api.list_deepnets,
+                           "tags__in=%s" % command_args.deepnet_tag))
+        command_args.deepnet_ids_ = deepnet_ids
+    except AttributeError:
+        pass
+
+    topic_model_ids = []
+    try:
+        # Parses topicmodel/ids if provided.
+        if command_args.topic_models:
+            topic_model_ids = u.read_resources(command_args.topic_models)
+        command_args.topic_model_ids_ = topic_model_ids
+    except AttributeError:
+        pass
+
+    # Retrieve topicmodel/ids if provided.
+    try:
+        if command_args.topic_model_tag:
+            topic_model_ids = (topic_model_ids +
+                               u.list_ids(api.list_topic_models,
+                                          "tags__in=%s" %
+                                          command_args.topic_model_tag))
+        command_args.topic_model_ids_ = topic_model_ids
+    except AttributeError:
+        pass
+
+
+    time_series_ids = []
+    try:
+        # Parses timeseries/ids if provided.
+        if command_args.time_series_set:
+            time_series_ids = u.read_resources(command_args.time_series)
+        command_args.time_series_ids_ = time_series_ids
+    except AttributeError:
+        pass
+
+    # Retrieve timeseries/ids if provided.
+    try:
+        if command_args.time_series_tag:
+            time_series_ids = (time_series_ids +
+                               u.list_ids(api.list_time_series,
+                                          "tags__in=%s" %
+                                          command_args.time_series_tag))
+        command_args.time_series_ids_ = time_series_ids
+    except AttributeError:
+        pass
+
+    pca_ids = []
+    try:
+        # Parses pca/ids if provided.
+        if command_args.pcas:
+            pca_ids = u.read_resources(command_args.pcas)
+        command_args.pca_ids_ = pca_ids
+    except AttributeError:
+        pass
+
+    # Retrieve pca/ids if provided.
+    try:
+        if command_args.pca_tag:
+            pca_ids = (pca_ids +
+                       u.list_ids(api.pca_series,
+                                  "tags__in=%s" %
+                                  command_args.pca_tag))
+        command_args.pca_ids_ = pca_ids
+    except AttributeError:
+        pass
+
+    # Parses models list for fusions if provided.
+    try:
+        if command_args.fusion_models:
+            fusion_models_arg = [
+                model.strip() for model in command_args.fusion_models.split(
+                    command_args.args_separator)]
+            command_args.fusion_models_ = fusion_models_arg
+        else:
+            command_args.fusion_models_ = []
+    except AttributeError:
+        pass
+
+    # Parses models list for fusions if provided.
+    if not has_value(command_args, "fusion_models_"):
+        try:
+            if command_args.fusion_models_file:
+                fusion_models_arg = u.read_json(
+                    command_args.fusion_models_file)
+                command_args.fusion_models_ = fusion_models_arg
+            else:
+                command_args.fusion_models_ = []
+        except AttributeError:
+            pass
+
+    fusion_ids = []
+    try:
+        # Parses fusion/ids if provided.
+        if command_args.fusions:
+            fusion_ids = u.read_resources(command_args.fusions)
+        command_args.fusion_ids_ = fusion_ids
+    except AttributeError:
+        pass
+
+    # Retrieve fusion/ids if provided.
+    try:
+        if command_args.fusion_tag:
+            fusion_ids = (fusion_ids +
+                          u.list_ids(api.fusion_series,
+                                     "tags__in=%s" %
+                                     command_args.fusion_tag))
+        command_args.fusion_ids_ = fusion_ids
+    except AttributeError:
+        pass
+
     # Parses cluster names to generate datasets if provided
     try:
         if command_args.cluster_datasets:
@@ -598,6 +951,73 @@ def get_output_args(api, command_args, resume):
     except AttributeError:
         pass
 
+    # if boosting arguments are used, set on boosting
+    try:
+        if command_args.iterations or command_args.learning_rate \
+                or command_args.early_holdout:
+            command_args.boosting = True
+    except AttributeError:
+        pass
+
+    # Extracts the imports from the JSON metadata file
+    try:
+        if command_args.embedded_imports:
+            command_args.embedded_imports_ = u.read_resources( \
+                command_args.embedded_imports)
+        else:
+            command_args.embedded_imports_ = []
+    except AttributeError:
+        pass
+
+    # Parses hidden_layers for deepnets.
+    try:
+        if command_args.hidden_layers:
+            command_args.hidden_layers_ = u.read_json(
+                command_args.hidden_layers)
+        else:
+            command_args.hidden_layers_ = []
+    except AttributeError:
+        pass
+
+    # Parses operating_point for predictions.
+    try:
+        if command_args.operating_point:
+            command_args.operating_point_ = u.read_json(
+                command_args.operating_point)
+        else:
+            command_args.operating_point_ = []
+    except AttributeError:
+        pass
+
+    # Parses the json_query
+    try:
+        if command_args.json_query:
+            command_args.json_query_ = u.read_json(command_args.json_query)
+        else:
+            command_args.json_query_ = None
+    except AttributeError:
+        pass
+
+    # Parses the models_file
+    try:
+        if command_args.models_file:
+            command_args.models_file_ = u.read_json(command_args.models_file)
+        else:
+            command_args.models_file_ = None
+    except AttributeError:
+        pass
+
+    # Parses the sql_output_fields
+    try:
+        if command_args.sql_output_fields:
+            command_args.sql_output_fields_ = u.read_json( \
+                command_args.sql_output_fields)
+        else:
+            command_args.sql_output_fields_ = None
+    except AttributeError:
+        pass
+
+
     return {"api": api, "args": command_args}
 
 
@@ -618,47 +1038,87 @@ def attribute_args(command_args):
             command_args.json_args[resource_type] = {}
 
 
-def transform_args(command_args, flags, api, user_defaults):
+def transform_args(command_args, flags, api):
     """Transforms the formatted argument strings into structured arguments
 
     """
     attribute_args(command_args)
 
     # Parses dataset generators in json format if provided
-    if command_args.new_fields:
-        json_generators = u.read_json(command_args.new_fields)
-        command_args.dataset_json_generators = json_generators
-    else:
-        command_args.dataset_json_generators = {}
+    try:
+        if command_args.new_fields:
+            json_generators = u.read_json(command_args.new_fields)
+            command_args.dataset_json_generators = json_generators
+        else:
+            command_args.dataset_json_generators = {}
+    except AttributeError:
+        pass
 
     # Parses multi-dataset attributes in json such as field maps
-    if command_args.multi_dataset_attributes:
-        multi_dataset_json = u.read_json(command_args.multi_dataset_attributes)
-        command_args.multi_dataset_json = multi_dataset_json
-    else:
-        command_args.multi_dataset_json = {}
+    try:
+        if command_args.multi_dataset_attributes:
+            multi_dataset_json = u.read_json(
+                command_args.multi_dataset_attributes)
+            command_args.multi_dataset_json = multi_dataset_json
+        else:
+            command_args.multi_dataset_json = {}
+    except AttributeError:
+        pass
 
     transform_dataset_options(command_args, api)
 
+    script_ids = None
+    command_args.script_ids = []
+    # Parses script/id if provided.
+    try:
+        if command_args.scripts:
+            script_ids = u.read_resources(command_args.scripts)
+            if len(script_ids) == 1:
+                command_args.script = script_ids[0]
+            command_args.script_ids = script_ids
+    except AttributeError:
+        pass
+
+    # Retrieve script/ids if provided.
+    try:
+        if command_args.script_tag:
+            script_ids = script_ids.extend(
+                u.list_ids(api.list_scripts,
+                           "tags__in=%s" % command_args.script_tag))
+            if len(script_ids) == 1:
+                command_args.script = script_ids[0]
+            command_args.script_ids = script_ids
+    except AttributeError:
+        pass
+
     # Reads a json filter if provided.
-    if command_args.json_filter:
-        json_filter = u.read_json_filter(command_args.json_filter)
-        command_args.json_filter = json_filter
+    try:
+        if command_args.json_filter:
+            json_filter = u.read_json_filter(command_args.json_filter)
+            command_args.json_filter = json_filter
+    except AttributeError:
+        pass
 
     # Reads a lisp filter if provided.
-    if command_args.lisp_filter:
-        lisp_filter = u.read_lisp_filter(command_args.lisp_filter)
-        command_args.lisp_filter = lisp_filter
+    try:
+        if command_args.lisp_filter:
+            lisp_filter = u.read_lisp_filter(command_args.lisp_filter)
+            command_args.lisp_filter = lisp_filter
+    except AttributeError:
+        pass
 
-    # Adds default tags unless it is requested not to do so.
-    if command_args.no_tag:
-        command_args.tag.append('BigMLer')
-        command_args.tag.append('BigMLer_%s' % NOW)
+    # Adds default tags unless that it is requested not to do so.
+    try:
+        if command_args.no_tag:
+            command_args.tag.append('BigMLer')
+            command_args.tag.append('BigMLer_%s' % NOW)
+    except AttributeError:
+        pass
 
     # Checks combined votes method
     try:
         if (command_args.method and command_args.method != COMBINATION_LABEL
-                and not (command_args.method in COMBINATION_WEIGHTS.keys())):
+                and not command_args.method in COMBINATION_WEIGHTS.keys()):
             command_args.method = 0
         else:
             combiner_methods = dict(
@@ -680,19 +1140,6 @@ def transform_args(command_args, flags, api, user_defaults):
     except AttributeError:
         pass
 
-    # Adds replacement=True if creating ensemble and nothing is specified
-    try:
-        if (command_args.number_of_models > 1 and
-                not command_args.replacement and
-                not '--no-replacement' in flags and
-                not 'replacement' in user_defaults and
-                not '--no-randomize' in flags and
-                not 'randomize' in user_defaults and
-                not '--sample-rate' in flags and
-                not 'sample_rate' in user_defaults):
-            command_args.replacement = True
-    except AttributeError:
-        pass
     try:
         # Old value for --prediction-info='full data' maps to 'full'
         if command_args.prediction_info == 'full data':
@@ -721,11 +1168,7 @@ def transform_args(command_args, flags, api, user_defaults):
         pass
 
     # Sets shared_flag if --shared or --unshared has been used
-    if '--shared' in flags or '--unshared' in flags:
-        command_args.shared_flag = True
-    else:
-        command_args.shared_flag = False
-
+    command_args.shared_flag = '--shared' in flags or '--unshared' in flags
 
     # Set remote on if scoring a trainind dataset in bigmler anomaly
     try:
@@ -736,25 +1179,40 @@ def transform_args(command_args, flags, api, user_defaults):
     except AttributeError:
         pass
 
-
-    command_args.has_models_ = (
+    command_args.has_supervised_ = (
         (hasattr(command_args, 'model') and command_args.model) or
         (hasattr(command_args, 'models') and command_args.models) or
         (hasattr(command_args, 'ensemble') and command_args.ensemble) or
         (hasattr(command_args, 'ensembles') and command_args.ensembles) or
+        (hasattr(command_args, 'model_tag') and command_args.model_tag) or
+        (hasattr(command_args, 'logistic_regression') and
+         command_args.logistic_regression) or
+        (hasattr(command_args, 'logistic_regressions') and
+         command_args.logistic_regressions) or
+        (hasattr(command_args, 'logistic_regression_tag') and
+         command_args.logistic_regression_tag) or
+        (hasattr(command_args, 'deepnet') and
+         command_args.deepnet) or
+        (hasattr(command_args, 'deepnets') and
+         command_args.deepnets) or
+        (hasattr(command_args, 'deepnet_tag') and
+         command_args.deepnet_tag) or
+        (hasattr(command_args, 'ensemble_tag')
+         and command_args.ensemble_tag))
+
+    command_args.has_models_ = (
+        command_args.has_supervised_ or
         (hasattr(command_args, 'cluster') and command_args.cluster) or
         (hasattr(command_args, 'clusters') and command_args.clusters) or
-        (hasattr(command_args, 'model_tag') and command_args.model_tag) or
         (hasattr(command_args, 'anomaly') and command_args.anomaly) or
         (hasattr(command_args, 'anomalies') and command_args.anomalies) or
-        (hasattr(command_args, 'ensemble_tag')
-         and command_args.ensemble_tag) or
         (hasattr(command_args, 'cluster_tag') and command_args.cluster_tag) or
         (hasattr(command_args, 'anomaly_tag') and command_args.anomaly_tag))
 
     command_args.has_datasets_ = (
         (hasattr(command_args, 'dataset') and command_args.dataset) or
         (hasattr(command_args, 'datasets') and command_args.datasets) or
+        (hasattr(command_args, 'dataset_ids') and command_args.dataset_ids) or
         (hasattr(command_args, 'dataset_tag') and command_args.dataset_tag))
 
 
@@ -766,28 +1224,57 @@ def transform_args(command_args, flags, api, user_defaults):
         (hasattr(command_args, 'test_dataset_tag') and
          command_args.test_dataset_tag))
 
+    command_args.new_dataset = (
+        (hasattr(command_args, 'datasets_json') and
+        command_args.datasets_json) or
+        (hasattr(command_args, 'multi_dataset') and
+        command_args.multi_dataset) or
+        (hasattr(command_args, 'juxtapose') and command_args.juxtapose) or
+        (hasattr(command_args, 'sql_query') and command_args.sql_query) or
+        (hasattr(command_args, 'sql_output_fields') and
+        command_args.sql_output_fields) or
+        (hasattr(command_args, 'json_query') and
+         command_args.json_query))
+
+
 def transform_dataset_options(command_args, api):
     """Retrieves the dataset ids from the different input options
 
     """
-    dataset_ids = None
     command_args.dataset_ids = []
-    # Parses dataset/id if provided.
-    if command_args.datasets:
-        dataset_ids = u.read_datasets(command_args.datasets)
-        if len(dataset_ids) == 1:
-            command_args.dataset = dataset_ids[0]
-        command_args.dataset_ids = dataset_ids
+    command_args.test_dataset_ids = []
+
+    try:
+        dataset_ids = None
+        # Parses dataset/id if provided.
+        if command_args.datasets:
+            dataset_ids = u.read_datasets(command_args.datasets)
+            if len(dataset_ids) > 0:
+                command_args.dataset = dataset_ids[-1]
+            command_args.dataset_ids = dataset_ids
+    except Exception:
+        pass
 
     # Reading test dataset ids is delayed till the very moment of use to ensure
     # that the newly generated resources files can be used there too
-    command_args.test_dataset_ids = []
+    try:
+        # Retrieve dataset/ids if provided.
+        if command_args.dataset_tag:
+            dataset_ids = dataset_ids.extend(
+                u.list_ids(api.list_datasets,
+                           "tags__in=%s" % command_args.dataset_tag))
+            if len(dataset_ids) > 0:
+                command_args.dataset = dataset_ids[-1]
+            command_args.dataset_ids = dataset_ids
+    except Exception:
+        pass
 
-    # Retrieve dataset/ids if provided.
-    if command_args.dataset_tag:
-        dataset_ids = dataset_ids.extend(
-            u.list_ids(api.list_datasets,
-                       "tags__in=%s" % command_args.dataset_tag))
-        if len(dataset_ids) == 1:
-            command_args.dataset = dataset_ids[0]
-        command_args.dataset_ids = dataset_ids
+    # if datasets_json is set, read it info in datasets
+    try:
+        if hasattr(command_args, 'datasets_json') \
+                and command_args.datasets_json:
+            command_args.dataset_ids = json.loads(command_args.datasets_json)
+            if len(dataset_ids) > 0:
+                command_args.dataset = command_args.dataset_ids[-1]
+    except AttributeError:
+        pass
