@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -24,13 +24,14 @@ import re
 import os
 
 import bigml.api
-import bigmler.utils as u
-import bigmler.resources as r
-import bigmler.checkpoint as c
 
 from bigml.fields import Fields
 from bigml.predicate import TM_FULL_TERM
 
+import bigmler.utils as u
+import bigmler.resources as r
+import bigmler.checkpoint as c
+
 from bigmler.prediction import OTHER
 
 
@@ -169,15 +170,17 @@ def dataset_processing(source, api, args, resume,
         dataset = r.create_dataset(source, dataset_args, args, api,
                                    path, session_file, log)
 
-    # If a dataset is provided, let's retrieve it.
-    elif args.dataset:
-        dataset = bigml.api.get_dataset_id(args.dataset)
-
     # If set of datasets is provided, let's check their ids.
     elif args.dataset_ids:
         for i in range(0, len(args.dataset_ids)):
-            datasets.append(bigml.api.get_dataset_id(args.dataset_ids[i]))
+            dataset_id = args.dataset_ids[i]
+            if isinstance(dataset_id, dict) and "id" in dataset_id:
+                dataset_id = dataset_id["id"]
+            datasets.append(bigml.api.get_dataset_id(dataset_id))
         dataset = datasets[0]
+    # If a dataset is provided, let's retrieve it.
+    elif args.dataset:
+        dataset = bigml.api.get_dataset_id(args.dataset)
 
     # If we already have a dataset, we check the status and get the fields if
     # we hadn't them yet.
@@ -282,6 +285,39 @@ def split_processing(dataset, api, args, resume,
     return train_dataset, test_dataset, resume
 
 
+def split_range_processing(dataset, api, args, resume,
+                           multi_label_data=None, session_file=None,
+                           path=None, log=None):
+    """Splits a dataset into train and test datasets using ranges
+
+    """
+    train_dataset = None
+    test_dataset = None
+    test_rate = args.test_split
+    train_rate = 1 - test_rate
+    split_row = int(dataset["object"]["rows"] * train_rate)
+    args.range_ = [1, split_row]
+    args.test_split = 0
+    dataset_alternative_args = r.set_dataset_split_args(
+        "%s - train (%s %%)" % (
+            args.name, int(train_rate * 100)),
+        args.description_, args,
+        multi_label_data=multi_label_data)
+    train_dataset, resume = alternative_dataset_processing(
+        dataset, "train", dataset_alternative_args, api, args,
+        resume, session_file=session_file, path=path, log=log)
+    args.range_ = [split_row + 1, dataset["object"]["rows"]]
+    dataset_alternative_args = r.set_dataset_split_args(
+        "%s - test (%s %%)" % (
+            args.name, int(test_rate * 100)),
+        args.description_, args, multi_label_data=multi_label_data)
+    test_dataset, resume = alternative_dataset_processing(
+        dataset, "test", dataset_alternative_args, api, args,
+        resume, session_file=session_file, path=path, log=log)
+
+    return train_dataset, test_dataset, resume
+
+
 def create_categories_datasets(dataset, distribution,
                                fields, args, api, resume,
                                session_file=None, path=None, log=None,
