@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -45,9 +45,10 @@ def ensemble_processing(datasets, api, args, resume,
         resume, ensemble_ids = c.checkpoint(
             c.are_ensembles_created, path, number_of_ensembles,
             debug=args.debug)
-        _, model_ids = c.checkpoint(c.are_models_created, path,
-            number_of_ensembles * args.number_of_models)
-        models = model_ids
+        if args.number_of_models > 1:
+            _, model_ids = c.checkpoint(c.are_models_created, path, \
+                number_of_ensembles * args.number_of_models)
+            models = model_ids
         if not resume:
             message = u.dated("Found %s ensembles out of %s. Resuming.\n"
                               % (len(ensemble_ids),
