@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2014-2016 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -58,8 +58,8 @@ def i_create_output(step, output=None, language=None, resource_type='source',
     world.folders.append(world.directory)
     resource_id = getattr(world, resource_type)['resource']
     try:
-        command = (u"bigmler reify --id " + resource_id +
-                   u" --store --output " + output)
+        command = (u"bigmler reify --id " + resource_id + " --language " +
+                   language + u" --store --output " + output)
         if add_fields:
             command += u' --add-fields'
         command = check_debug(command)
@@ -80,58 +80,78 @@ def i_check_output_file(step, output=None, check_file=None):
     if check_file is None or output is None:
         assert False
     check_file = res_filename(check_file)
-    output_file = os.path.join(world.directory, "reify.py")
+    output_file = os.path.join(world.directory, os.path.basename(output))
     with open(check_file, open_mode("r")) as check_file_handler:
         check_contents = check_file_handler.read().strip("\n")
+    """
         check_contents_lines = check_contents.split("\n")
         for index, line in enumerate(check_contents_lines):
             if line:
                 check_contents_lines[index] = INDENT + line
         check_contents = "\n".join(check_contents_lines)
+    """
     # remove unicode mark for strings if Python3
     if PYTHON3:
         check_contents = check_contents.replace( \
             " u'", " '").replace("{u'", "{'").replace( \
             ' u"', ' "').replace('u\\\'', '\\\'')
+        check_contents = re.sub(r'\n\s*', '\n', check_contents)
     with open(output_file, open_mode("r")) as output_file:
         output_file_contents = output_file.read()
+
     #strip comments at the beginning of the file
     output_file_contents = re.sub(r'#!.*def\smain\(\):\n', '',
                                   output_file_contents,
                                   flags=re.S).strip("\n")
     output_file_contents = output_file_contents.replace( \
-        '\n\nif __name__ == "__main__":\n    main()', '')
+        '\nif __name__ == "__main__":\n    main()', '')
 
     #strip internally added project id information
     prefix = "" if PYTHON3 else "u"
-    p_str = r',\s\\\n%s\{\'project\':\s%s\'project/[a-f0-9]{24}\'\}\)' \
-        % (INDENT * 2, prefix)
-    output_file_contents = re.sub(p_str,
-                                  ')', output_file_contents,
-                                  flags=re.S).strip("\n")
-    p_str = r',\s\\\n%s\s\'project\':\s%s\'project/[a-f0-9]{24}\'\}\)' \
-        % (INDENT * 2, prefix)
-    output_file_contents = re.sub(p_str,
-                                  ')', output_file_contents,
-                                  flags=re.S).strip("\n")
-    p_str = r',\n%s\s\'project\':\s%s\'project/[a-f0-9]{24}\'\}\)' \
-        % (INDENT * 2, prefix)
-    output_file_contents = re.sub(p_str,
-                                  '})', output_file_contents,
-                                  flags=re.S).strip("\n")
-    p_str = r',\s\'project\':\s%s\'project/[a-f0-9]{24}\'' % prefix
+    p_str = r'%s\'project\':\s%s\'project/[a-f0-9]{24}\',?\s?' \
+        % (prefix, prefix)
     output_file_contents = re.sub(p_str,
                                   '', output_file_contents,
                                   flags=re.S).strip("\n")
-    if check_contents == output_file_contents:
-        assert True
-    else:
+    p_str = r'/[a-f0-9]{24}'
+    output_file_contents = re.sub(p_str,
+                                  '', output_file_contents,
+                                  flags=re.S)
+    check_contents = re.sub(p_str,
+                            '', check_contents,
+                            flags=re.S)
+    p_str = r';;.*\n'
+    output_file_contents = re.sub(p_str,
+                                  '', output_file_contents,
+                                  flags=re.S)
+    check_contents = re.sub(p_str,
+                            '', check_contents,
+                            flags=re.S)
+    p_str = r'created by.*\n'
+    output_file_contents = re.sub(p_str,
+                                  '', output_file_contents,
+                                  flags=re.S)
+    check_contents = re.sub(p_str,
+                            '', check_contents,
+                            flags=re.S)
+    p_str = r'    api = .*?\n'
+    output_file_contents = re.sub(p_str,
+                                  '    api = BigML()\n', output_file_contents,
+                                  flags=re.S).strip("\n")
+    output_file_contents = re.sub(r'\n\s*', '\n', output_file_contents)
+    check_contents = re.sub(r'\n\s*', '\n', check_contents)
+    output_file_contents = output_file_contents.strip("\n").strip()
+    check_contents = check_contents.strip("\n").strip()
+    if check_contents != output_file_contents:
         if PYTHON3:
             # look for an alternative in PYTHON3
-            check_contents = python3_contents(check_file, check_contents)
+            check_contents = python3_contents( \
+                check_file, check_contents)
             if check_contents != output_file_contents:
                 check_contents = python3_contents(
                     check_file, check_contents, alternative="_1")
+        with open("%s_bck" % check_file, "w") as bck_file:
+            bck_file.write(output_file_contents)
         eq_(check_contents, output_file_contents)
 
 
@@ -148,7 +168,8 @@ def create_source(filename, output=None, args=None):
 
 #@step(r'I create a BigML dataset from a source with data "(.*)" and params "(.*)"')
 def create_dataset(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -160,7 +181,8 @@ def create_dataset(filename, output=None, args=None):
 
 #@step(r'I create a BigML dataset from dataset with data "(.*)" and params "(.*)"')
 def create_dataset_from_dataset(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -173,9 +195,31 @@ def create_dataset_from_dataset(filename, output=None, args=None):
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
 
+#@step(r'I create a BigML dataset from a list of datasets with data "(.*)" and params "(.*)"')
+def create_dataset_from_datasets(filename, output=None, args=None):
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
+    world.source = source
+    world.directory = os.path.dirname(output)
+    world.output = output
+    world.api.ok(world.source)
+    world.sources.append(source['resource'])
+    world.dataset = world.api.create_dataset(source)
+    world.api.ok(world.dataset)
+    world.datasets.append(world.dataset['resource'])
+    datasets = [world.dataset['resource']]
+    world.dataset = world.api.create_dataset(source)
+    world.api.ok(world.dataset)
+    world.datasets.append(world.dataset['resource'])
+    datasets.append(world.dataset)
+    world.dataset = world.api.create_dataset(datasets, args)
+    world.api.ok(world.dataset)
+    world.datasets.append(world.dataset['resource'])
+
 #@step(r'I create a BigML model from a dataset with data "(.*)" and params "(.*)"')
 def create_model(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -190,7 +234,8 @@ def create_model(filename, output=None, args=None):
 
 #@step(r'I create a BigML prediction for (.*) from a model with data "(.*)" and params "(.*)"')
 def create_prediction(filename, input_data= None, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -202,13 +247,15 @@ def create_prediction(filename, input_data= None, output=None, args=None):
     world.model = world.api.create_model(world.dataset)
     world.api.ok(world.model)
     world.models.append(world.model['resource'])
-    world.prediction = world.api.create_prediction(world.model, input_data, args)
+    world.prediction = world.api.create_prediction( \
+        world.model, input_data, args)
     world.api.ok(world.prediction)
     world.predictions.append(world.prediction['resource'])
 
 #@step(r'I create a BigML cluster from a dataset with data "(.*)" and params "(.*)"')
 def create_cluster(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -217,13 +264,15 @@ def create_cluster(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
+    args.update({"cluster_seed": "bigml"})
     world.cluster = world.api.create_cluster(world.dataset, args)
     world.api.ok(world.cluster)
     world.clusters.append(world.cluster['resource'])
 
 #@step(r'I create a BigML anomaly from a dataset with data "(.*)" and params "(.*)"')
 def create_anomaly(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -232,13 +281,15 @@ def create_anomaly(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
+    args.update({"seed": "bigml", "anomaly_seed": "bigml"})
     world.anomaly = world.api.create_anomaly(world.dataset, args)
     world.api.ok(world.anomaly)
     world.anomalies.append(world.anomaly['resource'])
 
 #@step(r'I create a BigML centroid for (.*) from a cluster with data "(.*)" and params "(.*)"')
 def create_centroid(filename, input_data=None, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -247,16 +298,19 @@ def create_centroid(filename, input_data=None, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.cluster = world.api.create_cluster(world.dataset)
+    world.cluster = world.api.create_cluster( \
+        world.dataset, {"cluster_seed": "bigml"})
     world.api.ok(world.cluster)
     world.clusters.append(world.cluster['resource'])
-    world.centroid = world.api.create_centroid(world.cluster, input_data, args)
+    world.centroid = world.api.create_centroid( \
+        world.cluster, input_data, args)
     world.api.ok(world.centroid)
     world.centroids.append(world.centroid['resource'])
 
 #@step(r'I create a BigML anomaly score for (.*) from an anomaly detector with data "(.*)" and params "(.*)"')
 def create_anomaly_score(filename, input_data=None, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -265,10 +319,12 @@ def create_anomaly_score(filename, input_data=None, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.anomaly = world.api.create_anomaly(world.dataset)
+    world.anomaly = world.api.create_anomaly( \
+        world.dataset, {"seed": "bigml", "anomaly_seed": "bigml"})
     world.api.ok(world.anomaly)
     world.anomalies.append(world.anomaly['resource'])
-    world.anomaly_score = world.api.create_anomaly_score(world.anomaly, input_data, args)
+    world.anomaly_score = world.api.create_anomaly_score( \
+        world.anomaly, input_data, args)
     world.api.ok(world.anomaly_score)
     world.anomaly_scores.append(world.anomaly_score['resource'])
 
@@ -286,13 +342,15 @@ def create_batch_prediction(filename, output=None, args=None):
     world.model = world.api.create_model(world.dataset)
     world.api.ok(world.model)
     world.models.append(world.model['resource'])
-    world.batch_prediction = world.api.create_batch_prediction(world.model, world.dataset, args)
+    world.batch_prediction = world.api.create_batch_prediction( \
+        world.model, world.dataset, args)
     world.api.ok(world.batch_prediction)
     world.batch_predictions.append(world.batch_prediction['resource'])
 
 #@step(r'I create a BigML batch centroid from a cluster with data "(.*)" and params "(.*)"')
 def create_batch_centroid(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -301,16 +359,19 @@ def create_batch_centroid(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.cluster = world.api.create_cluster(world.dataset)
+    world.cluster = world.api.create_cluster( \
+        world.dataset, {"cluster_seed": "bigml"})
     world.api.ok(world.cluster)
     world.clusters.append(world.cluster['resource'])
-    world.batch_centroid = world.api.create_batch_centroid(world.cluster, world.dataset, args)
+    world.batch_centroid = world.api.create_batch_centroid( \
+        world.cluster, world.dataset, args)
     world.api.ok(world.batch_centroid)
     world.batch_centroids.append(world.batch_centroid['resource'])
 
 #@step(r'I create a BigML batch anomaly score from an anomaly detector with data "(.*)" and params "(.*)"')
 def create_batch_anomaly_score(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -319,16 +380,19 @@ def create_batch_anomaly_score(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.anomaly = world.api.create_anomaly(world.dataset)
+    world.anomaly = world.api.create_anomaly( \
+        world.dataset, {"seed": "bigml", "anomaly_seed": "bigml"})
     world.api.ok(world.anomaly)
     world.anomalies.append(world.anomaly['resource'])
-    world.batch_anomaly_score = world.api.create_batch_anomaly_score(world.anomaly, world.dataset, args)
+    world.batch_anomaly_score = world.api.create_batch_anomaly_score( \
+        world.anomaly, world.dataset, args)
     world.api.ok(world.batch_anomaly_score)
     world.batch_anomaly_scores.append(world.batch_anomaly_score['resource'])
 
 #@step(r'I create a BigML evaluation with data "(.*)" and params "(.*)"')
 def create_evaluation(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -340,14 +404,16 @@ def create_evaluation(filename, output=None, args=None):
     world.model = world.api.create_model(world.dataset)
     world.api.ok(world.model)
     world.models.append(world.model['resource'])
-    world.evaluation = world.api.create_evaluation(world.model, world.dataset, args)
+    world.evaluation = world.api.create_evaluation( \
+        world.model, world.dataset, args)
     world.api.ok(world.evaluation)
     world.evaluations.append(world.evaluation['resource'])
 
 
 #@step(r'I create a BigML ensemble from a dataset with data "(.*)" and params "(.*)"')
 def create_ensemble(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -356,13 +422,15 @@ def create_ensemble(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
+    args.update({"seed": "BigML", "ensemble_sample": {"seed": "bigml"}})
     world.ensemble = world.api.create_ensemble(world.dataset, args)
     world.api.ok(world.ensemble)
     world.ensembles.append(world.ensemble['resource'])
 
 #@step(r'I create a BigML evaluation with data "(.*)" split training/test and params "(.*)"')
 def create_evaluation_split(filename, output=None, args=None):
-    source = world.api.create_source(res_filename(filename), {"project": world.project_id})
+    source = world.api.create_source( \
+        res_filename(filename), {"project": world.project_id})
     world.source = source
     world.directory = os.path.dirname(output)
     world.output = output
@@ -371,16 +439,19 @@ def create_evaluation_split(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.dataset_train = world.api.create_dataset(world.dataset, {'sample_rate': 0.7})
+    world.dataset_train = world.api.create_dataset(world.dataset, { \
+        'sample_rate': 0.7, 'seed': 'BigML'})
     world.api.ok(world.dataset_train)
     world.datasets.append(world.dataset_train['resource'])
-    world.dataset_test = world.api.create_dataset(world.dataset, {'sample_rate': 0.7, 'out_of_bag': True})
+    world.dataset_test = world.api.create_dataset(world.dataset, { \
+        'sample_rate': 0.7, 'seed': 'BigML', 'out_of_bag': True})
     world.api.ok(world.dataset_test)
     world.datasets.append(world.dataset_test['resource'])
     world.model = world.api.create_model(world.dataset_train)
     world.api.ok(world.model)
     world.models.append(world.model['resource'])
-    world.evaluation = world.api.create_evaluation(world.model, world.dataset_test, args)
+    world.evaluation = world.api.create_evaluation( \
+        world.model, world.dataset_test, args)
     world.api.ok(world.evaluation)
     world.evaluations.append(world.evaluation['resource'])
 
@@ -398,13 +469,15 @@ def create_dataset_from_batch_prediction(filename, output=None, args=None):
     world.model = world.api.create_model(world.dataset)
     world.api.ok(world.model)
     world.models.append(world.model['resource'])
-    world.batch_prediction = world.api.create_batch_prediction(world.model, world.dataset, {"output_dataset": True})
+    world.batch_prediction = world.api.create_batch_prediction( \
+        world.model, world.dataset, {"output_dataset": True})
     world.api.ok(world.batch_prediction)
     world.batch_predictions.append(world.batch_prediction['resource'])
     world.batch_prediction_dataset = world.api.get_dataset(
         world.batch_prediction['object']['output_dataset_resource'])
     world.api.ok(world.batch_prediction_dataset)
-    world.batch_prediction_dataset = world.api.update_dataset(world.batch_prediction_dataset, args)
+    world.batch_prediction_dataset = world.api.update_dataset( \
+        world.batch_prediction_dataset, args)
     world.api.ok(world.batch_prediction_dataset)
     world.datasets.append(world.batch_prediction_dataset['resource'])
 
@@ -419,16 +492,19 @@ def create_dataset_from_batch_centroid(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.cluster = world.api.create_cluster(world.dataset)
+    world.cluster = world.api.create_cluster \
+        (world.dataset, {"cluster_seed": "bigml"})
     world.api.ok(world.cluster)
     world.clusters.append(world.cluster['resource'])
-    world.batch_centroid = world.api.create_batch_centroid(world.cluster, world.dataset, {"output_dataset": True})
+    world.batch_centroid = world.api.create_batch_centroid( \
+        world.cluster, world.dataset, {"output_dataset": True})
     world.api.ok(world.batch_centroid)
     world.batch_centroids.append(world.batch_centroid['resource'])
     world.batch_centroid_dataset = world.api.get_dataset(
         world.batch_centroid['object']['output_dataset_resource'])
     world.api.ok(world.batch_centroid_dataset)
-    world.batch_centroid_dataset = world.api.update_dataset(world.batch_centroid_dataset, args)
+    world.batch_centroid_dataset = world.api.update_dataset( \
+        world.batch_centroid_dataset, args)
     world.api.ok(world.batch_centroid_dataset)
     world.datasets.append(world.batch_centroid_dataset['resource'])
 
@@ -443,16 +519,20 @@ def create_dataset_from_batch_anomaly(filename, output=None, args=None):
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.anomaly = world.api.create_anomaly(world.dataset)
+    world.anomaly = world.api.create_anomaly(world.dataset,
+                                             {"seed": "bigml",
+                                              "anomaly_seed": "bigml"})
     world.api.ok(world.anomaly)
     world.anomalies.append(world.anomaly['resource'])
-    world.batch_anomaly_score = world.api.create_batch_anomaly_score(world.anomaly, world.dataset, {"output_dataset": True})
+    world.batch_anomaly_score = world.api.create_batch_anomaly_score( \
+        world.anomaly, world.dataset, {"output_dataset": True})
     world.api.ok(world.batch_anomaly_score)
     world.batch_anomaly_scores.append(world.batch_anomaly_score['resource'])
     world.batch_anomaly_score_dataset = world.api.get_dataset(
         world.batch_anomaly_score['object']['output_dataset_resource'])
     world.api.ok(world.batch_anomaly_score_dataset)
-    world.batch_anomaly_score_dataset = world.api.update_dataset(world.batch_anomaly_score_dataset, args)
+    world.batch_anomaly_score_dataset = world.api.update_dataset( \
+        world.batch_anomaly_score_dataset, args)
     world.api.ok(world.batch_anomaly_score_dataset)
     world.datasets.append(world.batch_anomaly_score_dataset['resource'])
 
@@ -468,16 +548,19 @@ def create_dataset_from_dataset_from_batch_centroid(filename, output=None, args=
     world.dataset = world.api.create_dataset(source)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
-    world.cluster = world.api.create_cluster(world.dataset)
+    world.cluster = world.api.create_cluster( \
+        world.dataset, {"cluster_seed": "bigml"})
     world.api.ok(world.cluster)
     world.clusters.append(world.cluster['resource'])
-    world.batch_centroid = world.api.create_batch_centroid(world.cluster, world.dataset, {"output_dataset": True})
+    world.batch_centroid = world.api.create_batch_centroid( \
+        world.cluster, world.dataset, {"output_dataset": True})
     world.api.ok(world.batch_centroid)
     world.batch_centroids.append(world.batch_centroid['resource'])
     world.batch_centroid_dataset = world.api.get_dataset(
         world.batch_centroid['object']['output_dataset_resource'])
     world.api.ok(world.batch_centroid_dataset)
     world.datasets.append(world.batch_centroid_dataset['resource'])
-    world.dataset = world.api.create_dataset(world.batch_centroid_dataset['resource'], args)
+    world.dataset = world.api.create_dataset( \
+        world.batch_centroid_dataset['resource'], args)
     world.api.ok(world.dataset)
     world.datasets.append(world.dataset['resource'])
