@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2012-2016 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -141,6 +141,7 @@ def read_json(path):
         json_attributes = json.loads(attributes_reader.read())
     except ValueError:
         sys.exit("Error: no valid json found in %s" % path)
+    attributes_reader.close()
     return json_attributes
 
 
@@ -281,7 +282,7 @@ def read_local_resource(path, csv_properties=None):
 
     fields, resource_locale, missing_tokens, \
     objective_column = get_fields_structure(resource)
-    if not 'objective_field' in csv_properties and \
+    if 'objective_field' not in csv_properties and \
             objective_column is not None:
         csv_properties['objective_field'] = objective_column
     if missing_tokens:
@@ -293,40 +294,48 @@ def read_local_resource(path, csv_properties=None):
     return resource, csv_properties, fields
 
 
-def list_ids(api_function, query_string, status_code=bigml.api.FINISHED):
+def list_ids(api_function, query_string, status_code=bigml.api.FINISHED,
+             limit=None):
     """Lists BigML resources filtered by `query_string`.
 
     """
-    q_s = 'status.code=%s;limit=%s;%s' % (
-        status_code, PAGE_LENGTH, query_string)
+    limit_s = 'limit=%s' % (PAGE_LENGTH if limit is None else limit)
+    q_s = 'status.code=%s;%s;%s' % (
+        status_code, limit_s, query_string)
     resources = api_function(q_s)
     ids = [obj['resource'] for obj in (resources['objects'] or [])]
-    while (resources['objects'] and
+    while (resources['objects'] and (limit is None or len(ids) < limit) and
            (resources['meta']['total_count'] > (resources['meta']['offset'] +
                                                 resources['meta']['limit']))):
         offset = resources['meta']['offset'] + PAGE_LENGTH
         q_s = 'status.code=%s;offset=%s;limit=%s;%s' % (
-            status_code, offset, PAGE_LENGTH, query_string)
+            status_code, offset, limit_s, query_string)
         resources = api_function(q_s)
         if resources['objects']:
             ids.extend([obj['resource'] for obj in resources['objects']])
     return ids
 
 
-def delete(api, delete_list):
-    """ Deletes the resources given in the list.
+def delete(api, delete_list, exe_outputs=True):
+    """ Deletes the resources given in the list. If the exe_outputs is set,
+        deleting an execution causes the deletion of any outpur resource.
 
     """
     for resource_id in delete_list:
         resource_type = None
         try:
-            for resource_type in bigml.api.RESOURCE_RE.keys():
+            for resource_type in bigml.api.RESOURCE_RE:
                 try:
                     bigml.api.get_resource(resource_type, resource_id)
                     break
                 except ValueError:
                     pass
-            api.deleters[resource_type](resource_id)
+            if resource_type == "execution" and exe_outputs:
+                query_string = "delete_all=true"
+                api.deleters[resource_type](resource_id,
+                                            query_string=query_string)
+            else:
+                api.deleters[resource_type](resource_id)
         except ValueError:
             console_log("Failed to delete resource %s" % resource_id)
 
@@ -336,10 +345,13 @@ def check_dir(path):
 
     """
     directory = os.path.dirname(path)
+    if directory == "":
+        directory = "."
     if len(directory) > 0 and not os.path.exists(directory):
         os.makedirs(directory)
         sys_log_message(u"%s\n" % os.path.abspath(directory),
                         log_file=NEW_DIRS_LOG)
+
     return directory
 
 
@@ -443,9 +455,14 @@ def check_resource_error(resource, message):
     """
     if ('error' in resource and resource['error'] or
             bigml.api.get_status(resource)['code'] == bigml.api.FAULTY):
-        if ('status' in resource['error'] and
-                'message' in resource['error']['status']):
-            error_message = resource['error']['status']['message']
+        try:
+            if ('status' in resource['error'] and
+                    'message' in resource['error']['status']):
+                error_message = resource['error']['status']['message']
+        except TypeError:
+            status = bigml.api.get_status(resource)
+            if status.get('error') is not None:
+                error_message = status["message"]
         sys.exit("%s%s" % (message, error_message))
     return bigml.api.get_resource_id(resource)
 
@@ -477,9 +494,10 @@ def check_resource(*args, **kwargs):
 
     """
     try:
+        kwargs.update({"raise_on_error": True})
         result = bigml.api.check_resource(*args, **kwargs)
         return result
-    except ValueError, exc:
+    except Exception, exc:
         sys.exit("\nFailed to obtain a finished resource:\n%s." % str(exc))
 
 
@@ -707,3 +725,73 @@ def transform_fields_keys(json_attributes, fields):
             fields_structure[key] = json_attributes["fields"][old_key]
         json_attributes["fields"] = fields_structure
     return json_attributes
+
+
+def get_last_resource(resource_type, api=None, query_string=None):
+    """Retrieves the last resource that meets the conditions in args
+
+    """
+    if query_string is None:
+        query_string = ""
+    if api is None:
+        api = bigml.api.BigML()
+    ids = list_ids(api.listers[resource_type], query_string, limit=1)
+    if ids:
+        return ids[0]
+    return None
+
+
+def get_first_resource(resource_type, api=None, query_string=None):
+    """Retrieves the first resource that meets the conditions in args
+
+    """
+    order = "order_by=created"
+    if query_string is None:
+        query_string = order
+    else:
+        query_string = "%s;%s" % (query_string, order)
+
+    if api is None:
+        api = bigml.api.BigML()
+    ids = list_ids(api.listers[resource_type], query_string, limit=1)
+    if ids:
+        return ids[0]
+    return None
+
+
+def last_resource_url(resource_id, api=None, query_string=None):
+    """Creates the last resource URL of the given type
+    that meets the conditions in args
+
+    """
+    if query_string is None:
+        query_string = ""
+    if api is None:
+        api = bigml.api.BigML()
+    auth = "%sproject=%s;" % (api.auth, api.project) if api.project else api.auth
+    resource_type = bigml.api.get_resource_type(resource_id)
+    return "%s%s%s%s" % (api.url, resource_type, auth, query_string)
+
+
+def get_script_id(path):
+    """Returns the script id stored in the file in path
+
+    """
+    try:
+        with open(path) as file_handler:
+            return file_handler.read().strip()
+    except IOError:
+        return None
+
+
+def add_api_context(command_args, args):
+    """Extends the command_list with the arguments related to the api
+       credentials.
+
+    """
+    if args.org_project:
+        command_args.extend(['--org-project', args.org_project])
+    if args.username:
+        command_args.extend(['--username', args.username])
+    if args.api_key:
+        command_args.extend(['--api-key', args.api_key])
